/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "CNRParser.h"
#include "CNRTokens.h"

#include "CNR.h"
#include "CNR_HarvestRatioMultipleEncounter.h"
#include "CNR_SeasonLengthMultipleEncounter.h"
#include "HarvestManager.h"
#include "GenericArrayFactory.h"


// ctor utility fns
void
CNRParser::initKeymap()
{
    keymap_.insert(CNRTokens::CNRMETHOD, cnrMethod);
    keymap_.insert(CNRTokens::HARVESTRATIO, harvestRatio);
    keymap_.insert(CNRTokens::SEASONLENGTH, seasonLength);
    keymap_.insert(CNRTokens::REPORTEDENCOUNTER, reportedEncounter);
    keymap_.insert(CNRTokens::HARVESTRATIOMULTIPLEENCOUNTER, harvestRatioMultipleEncounter);
    keymap_.insert(CNRTokens::SEASONLENGTHMULTIPLEENCOUNTER, seasonLengthMultipleEncounter);
    keymap_.insert(CNRTokens::LEGALSELECTIVITY, legalSelectivity);
    keymap_.insert(CNRTokens::SUBLEGALSELECTIVITY, sublegalSelectivity);
    keymap_.insert(CNRTokens::LEGALSEASONLENGTH, legalSeasonLength);
    keymap_.insert(CNRTokens::CNRSEASONLENGTH, cnrSeasonLength);
    keymap_.insert(CNRTokens::LEGALENCOUNTERS, legalEncounters);
    keymap_.insert(CNRTokens::SUBLEGALENCOUNTERS, sublegalEncounters);
    keymap_.insert(CNRTokens::LANDEDCATCH, landedCatch);
    keymap_.insert(CNRTokens::LEGALRELEASEMORTRATE, releaseMortRate);
    keymap_.insert(CNRTokens::BASESEASON, baseSeason);
    keymap_.insert(CNRTokens::RECAPTUREINTERVAL, recaptureInterval);

    initGenArrayKeymap(keymap_);
}

void
CNRParser::initState()
{
    // configure the initial state with the correct id list.
    State* state = new State;
    state->context = "CNR_Data";
    configCNR(state);
    stateStack.push(state);
}

CNRParser::CNRParser(Parser& p, RWCString dimension)
    : GenArrayParser(p), dimension_(dimension)
{
    clearData();
    initState();
    initKeymap();
}

// create a generic array for cnr data of the requested dimension size
// and configure the HarvestManager with it.  also add the list of
// generic ids to index every element of the array to the current
// state.

void
CNRParser::configCNR(State* state)
{
    GenericArrayFactory<auto_ptr<CNR> > cnrFactory;
    GenericArrayFactory<auto_ptr<CNR> >::GenericArrayType arraytype = cnrFactory.genericArrayType(dimension_); 
    if (arraytype != cnrFactory.NTypes) {
        // table entries are null auto_ptrs by default
        CNRTable* cnrTable = cnrFactory.newGenericArrayFn0(arraytype); 
        HarvestManager::configCNRs(auto_ptr<CNRTable>(cnrTable)); // hand off storage to the manager
        cnrTable->newGenericIndexList(state->ids); // create the proper list of generic ids
    }
    else {
        // unidentified dimension.  log an error, but allow this
        // parser to continue with an empty id list in the state, so
        // as to skip as much data as possible without generating
        // further messages.
        logError(pError(ParseError::BadData,
            dimension_ + " is not a valid configuration for"
            + state->context + "; discarding data")
            .errorMessage());
    }
}

// clear local data
void CNRParser::
clearData()
{
    cnrType_ = CnrNone;
    legalSelectivity_ = 0;
    sublegalSelectivity_ = 0;
    legalSeasonLength_ = 0;
    cnrSeasonLength_ = 0;
    legalEncounters_ = 0;
    sublegalEncounters_ = 0;
    landedCatch_ = 0;
    baseSeason_ = 0;
    recaptureInterval_ = 0;
    legalReleaseMortRate_ = 0;
}

// upon reading an "end" token, if we have data for a new cnr object,
// make copies and place them in the HarvestManager
// at every generic id region in the current list.

void
CNRParser::parseEnd()
{
#ifdef DEBUG
    assert(cnrType_ >= CnrNone && cnrType_ < NumCnrTypes);
#endif

    // use the existence of a cnrType to decide whether we are ready to commit
    // a new cnr object.  if not, pop one level of state and continue.
    if (cnrType_ == CnrNone) {
        // call the parent class end fn to pop the state
        GenArrayParser::parseEnd();
    }
    else {
        // iterate through the ids in the current state
        RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
        while (iter != (stateStack.top()->ids).end()) {
            GenericArrayIndex* id = *iter;
            
            // make object, hand off the ptr to the harvest manager, which assumes ownership
            CNR* cnr;
            switch (cnrType_) {
            case CnrHarvestRatio:
                cnr = new CNR_HarvestRatio(legalSelectivity_, sublegalSelectivity_);
                break;
            case CnrSeasonLength:
                cnr = new CNR_SeasonLength(legalSelectivity_, sublegalSelectivity_,
                    legalSeasonLength_, cnrSeasonLength_);
                break;
            case CnrReportedEncounter:
                cnr = new CNR_ReportedEncounter(legalSelectivity_, sublegalSelectivity_,
                    legalEncounters_, sublegalEncounters_, landedCatch_);
                break;
            case CnrHarvestRatioMultipleEncounter:
                cnr = new CNR_HarvestRatioMultipleEncounter(legalSelectivity_, sublegalSelectivity_, 
                    legalReleaseMortRate_, baseSeason_, recaptureInterval_);
                break;
            case CnrSeasonLengthMultipleEncounter:
                cnr = new CNR_SeasonLengthMultipleEncounter(legalSelectivity_, sublegalSelectivity_,
                    legalSeasonLength_, cnrSeasonLength_, legalReleaseMortRate_, baseSeason_,
                    recaptureInterval_);
                break;
            }
            
            HarvestManager::insertCNR(auto_ptr<CNR>(cnr), *id);
            ++iter;
        }
        // clear the cnrType and all other data
        clearData();
        nextDataLine();		// skip the rest of this line as comments
    }
}

void CNRParser::cnrMethod()
{
    // prepare for new object data
    clearData();
}

void CNRParser::harvestRatio()
{
    cnrType_ = CnrHarvestRatio;
}

void CNRParser::seasonLength()
{
    cnrType_ = CnrSeasonLength;
}

void CNRParser::reportedEncounter()
{
    cnrType_ = CnrReportedEncounter;
}

void CNRParser::harvestRatioMultipleEncounter()
{
    cnrType_ = CnrHarvestRatioMultipleEncounter;
}

void CNRParser::seasonLengthMultipleEncounter()
{
    cnrType_ = CnrSeasonLengthMultipleEncounter;
}

void CNRParser::legalSelectivity()
{
    parseVar(legalSelectivity_);
}

void CNRParser::sublegalSelectivity()
{
    parseVar(sublegalSelectivity_);
}

void CNRParser::legalSeasonLength()
{
    parseVar(legalSeasonLength_);
}

void CNRParser::cnrSeasonLength()
{
    parseVar(cnrSeasonLength_);
}

void CNRParser::legalEncounters()
{
    parseVar(legalEncounters_);
}

void CNRParser::sublegalEncounters()
{
    parseVar(sublegalEncounters_);
}

void CNRParser::landedCatch()
{
    parseVar(landedCatch_);
}

void CNRParser::releaseMortRate()
{
    parseVar(legalReleaseMortRate_);
}

void CNRParser::baseSeason()
{
    parseVar(baseSeason_);
}

void CNRParser::recaptureInterval()
{
    parseVar(recaptureInterval_);
}
