/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "CeilingScalarParser.h"
#include "CeilingScalarConfigParser.h"
#include "MultiCeilingScalarPolicyControl.h"
#include "PolicyControlManager.h"
#include "CeilingScalarTokens.h"
#include "TopTokens.h"
#include "GenArrayTokens.h"
#include "Tokens.h"


// ctor utility fn

void CeilingScalarParser::initKeymap(){
    keymap_.insert(CeilingScalarTokens::BASESTART, baseStart);
    keymap_.insert(CeilingScalarTokens::BASEEND, baseEnd);
    keymap_.insert(CeilingScalarTokens::FIRSTSTEP, firstStep);
    keymap_.insert(CeilingScalarTokens::LASTSTEP, lastStep);
    keymap_.insert(GenArrayTokens::FISHERY, fishery);
    keymap_.insert(Tokens::END1, parseEndKeyword);
    keymap_.insert(Tokens::END2, parseEndKeyword);
}

CeilingScalarParser::CeilingScalarParser(Parser& p, bool multi) 
    : SubParser(p), multi_(multi), errorEncountered_(false)
{
    initKeymap();

    // create a CeilingScalarPolicyControl to be configured.
    // use a zero length season.  create a multiCeiling if requested.
    if (multi)
        ceilings_ = auto_ptr<CeilingScalarPolicyControl>(new MultiCeilingScalarPolicyControl(-1,-1,-1,-1));
    else
        ceilings_ = auto_ptr<CeilingScalarPolicyControl>(new CeilingScalarPolicyControl(-1,-1));
}

RWCString
CeilingScalarParser::context() const
{
    if (multi_)
        return(TopTokens::MULTICEILINGSCALAR);
    else
        return(TopTokens::CEILINGSCALAR);
}

void CeilingScalarParser::baseStart()
{
    int start;
    parseVar(start);
    ceilings_->basePeriodStart_ = start;
}

void CeilingScalarParser::baseEnd()
{
    int end;
    parseVar(end);
    ceilings_->basePeriodEnd_ = end;
}

// first and last timesteps of iteration for a multi-ceiling policy
void CeilingScalarParser::firstStep()
{
    int first;
    parseVar(first);
    // convert from user timestep to internal representation
    --first;
    // since this is a multiCeiling, downcast the PolicyControl
    MultiCeilingScalarPolicyControl* policy = 
        dynamic_cast<MultiCeilingScalarPolicyControl*> (ceilings_.get());
    if (policy) 
        policy->firstStep_ = first;
    else {
        logError(pError(ParseError::BadKey,
            "error, timesteps should only be specified for MultiTimeStepCeilings, discarding; "
            + context()).errorMessage());
        errorEncountered_ = true;
    }
}

void CeilingScalarParser::lastStep()
{
    int last;
    parseVar(last);
    // convert from user timestep to internal representation
    --last;
    // since this is a multiCeiling, downcast the PolicyControl
    MultiCeilingScalarPolicyControl* policy = 
        dynamic_cast<MultiCeilingScalarPolicyControl*> (ceilings_.get());
    if (policy) 
        policy->lastStep_ = last;
    else {
        logError(pError(ParseError::BadKey,
            "error, timesteps may only be specified for MultiTimeStepCeilings, discarding; "
            + context()).errorMessage());
        errorEncountered_ = true;
    }
}

// read a fishery name and start recording CeilingScalarConfig data
void CeilingScalarParser::fishery()
{
    RWCString fname;
    parseVar(fname);
    CeilingScalarConfigParser fisheryCeilingParser(*this, fname, ceilings_->ceilingConfigs_);
    fisheryCeilingParser.parse();
}

void CeilingScalarParser::parseEndKeyword()
{
    if (!ceilings_->valid()) {
        logError(pError(ParseError::BadData,
            "invalid configuration for ceiling scalars (discarding) - try checking base period specification; "
            + context()).errorMessage());
        errorEncountered_ = true;
    }
    if (!errorEncountered_) {
        // upcast via reassignment of autoptr 
        auto_ptr<PolicyControl> policy(ceilings_.release());
        PolicyControlManager::addPolicyControl(policy);
    }
    SubParser::parseEndKeyword();
}