/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

// CeilingScalarPolicyControl.cpp


#include "CeilingScalarPolicyControl.h"
#include "Chronograph.h"
#include "SystemClock.h"
#include "FisheryList.h"
#include "FisheryManager.h"
#include "FisheryQuotaPolicy.h"
#include "Historian.h"
#include "GenericArray.h"
#include "auto_ptr.h"


// valid state for this object is base years in range
bool CeilingScalarPolicyControl::
valid() const
{
    int baseStartIndex = basePeriodStart_ - Chronograph::baseYear();
    int baseEndIndex = basePeriodEnd_ - Chronograph::baseYear();
    if (baseStartIndex < 0 
        || baseEndIndex >= Chronograph::nYears()
        || baseStartIndex > baseEndIndex)
        return false;
    else
        return true;
}

// managementAction routine is where things get done.  this
// PolicyControl expects to be called after every harvest
// process invocation.

void CeilingScalarPolicyControl::
postHarvestManagementAction()
{
    // if the base period has just finished, create all the FisheryQuota objects
    if (::SystemClock->currentCalendarYear() == basePeriodEnd_ 
        && ::SystemClock->currentTimeIndex() == ::Chronograph::lastStep()) {
        
        for (int i=0; i < ceilingConfigs_.entries(); ++i) {
            createQuotas(ceilingConfigs_(i));
        }
    }
}

// ctor for configs - init all years to inactive
CeilingScalarPolicyControl::CeilingScalarConfig::
CeilingScalarConfig(const Fishery* f) 
    : fishery_(f), yearConfig_(Chronograph::nYears())
{
    for (int i=0; i<yearConfig_.length(); ++i) {
        yearConfig_(i).active_ = false;
    }
}


// create quotas for every active year, timestep, region for this fishery

void CeilingScalarPolicyControl::
createQuotas(const CeilingScalarConfig& config) const
{
    int baseStartIndex = basePeriodStart_ - Chronograph::baseYear();
    int baseEndIndex = basePeriodEnd_ - Chronograph::baseYear();
    const Fishery* fishery = config.fishery();

    for (int yr = 0; yr < config.years(); ++yr) {
        struct CeilingScalarConfig::YearInfo yearInfo = config.yearConfig(yr);
        if (yearInfo.active_) {
            for (int time = Chronograph::firstStep(); time < Chronograph::lastStep(); ++time) {
                for (int region=0; region < Geographer::nRegions(); ++region) {
                    double quota = yearInfo.scalar_ * Historian::basePeriodAverageCatch(fishery, region, time,
                        baseStartIndex, baseEndIndex);
                    auto_ptr<FisheryPolicy> policy(new FisheryQuotaPolicy(quota, yearInfo.force_));
                    GenericArrayIndex gendex(0, fishery->index(), region, yr, time);
                    FisheryManager::insertPolicy(policy, gendex);
                }
            }
        }
    }
}
