/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// Chronographer class.  keeps track of years and timesteps

#ifndef Chronograph_H
#define Chronograph_H

#include "Bool.h"

// Chronograph to handle time tracking.  

typedef unsigned int TimeStep;

class Chronograph {
  public:
    Chronograph() : timestep_(0) {}
    
    static void configTimeSteps(unsigned int nTimeSteps) { nTimeSteps_ = nTimeSteps; }
    static void configYears(const int nYears, const int baseYear)
    { nYears_ = nYears; baseCalendarYear_ = baseYear; }

    static void checkConfig();  // checks for proper configuration

    void resetClock() { currentYearIndex_ = 0; timestep_ = 0; }
    void resetClock(unsigned int yearIndex, unsigned int timeIndex) 
    { currentYearIndex_ = yearIndex; timestep_ = timeIndex; }
    void resetTimeSteps() { timestep_ = 0; }
    
    static void incrementYear() { ++currentYearIndex_; }
    void incrementTimeStep() { ++timestep_; }
    
    static bool yearExpired() { return (currentYearIndex_ >= nYears_); }
    bool timeExpired() { return (timestep_ >= nTimeSteps_); }
    
    static unsigned int nYears() { return nYears_; }
    static unsigned int nTimeSteps() { return nTimeSteps_; }
    static unsigned int firstStep() { return 0; } // first timestep is always 0
    static unsigned int lastStep() { return nTimeSteps_ - 1; }
    static unsigned int baseYear() { return baseCalendarYear_; } // first calendar year
    
    // current year as brood year
    static unsigned int currentCalendarYear() { return baseCalendarYear_ + currentYearIndex_; }
    
    // methods to access the current year and timestep indices
    static unsigned int currentYearIndex() { return currentYearIndex_; }
    unsigned int currentTimeIndex() const { return timestep_; }
    unsigned int elapsedTime() const { return currentYearIndex_ * nTimeSteps_ + timestep_; }
    
private:
    static unsigned int nYears_;          // number of years in simulation
    static unsigned int baseCalendarYear_;  // in 4 digit years
    static unsigned int currentYearIndex_;  // index of current simulation year
    static unsigned int nTimeSteps_;
    
    unsigned int timestep_;                 // the current timestep index
};

// a TimeStamp holds a year and timestep record.  it uses indices (as opposed
// to calendar years).

class TimeStamp {
  public:
    TimeStamp(unsigned int year, unsigned int timeStep)
      : year_(year), timeStep_(timeStep)
    {}
    TimeStamp(const Chronograph& clock)
      : year_(clock.currentYearIndex()), timeStep_(clock.currentTimeIndex())
    {}

    int operator== (const TimeStamp& rhs) const
    { return (year_ == rhs.year_ && timeStep_ == rhs.timeStep_); }
    int operator< (const TimeStamp& rhs) const
    { return (year_ < rhs.year_ || (year_ == rhs.year_ && timeStep_ < rhs.timeStep_)); }
    int operator> (const TimeStamp& rhs) const
    { return (year_ > rhs.year_ || (year_ == rhs.year_ && timeStep_ > rhs.timeStep_)); }
    int operator<= (const TimeStamp& rhs) const
    { return (year_ < rhs.year_ || (year_ == rhs.year_ && timeStep_ <= rhs.timeStep_)); }
    int operator>= (const TimeStamp& rhs) const
    { return (year_ > rhs.year_ || (year_ == rhs.year_ && timeStep_ >= rhs.timeStep_)); }

    const TimeStamp& operator++()
    { if (timeStep_ < Chronograph::lastStep()) {++timeStep_;} else {++year_; timeStep_=0;} return *this; }

    unsigned int yearIndex() const
    { return year_; }
    unsigned int timeIndex() const 
    { return timeStep_; }
    
    unsigned int elapsedTime() const
    { return year_ * Chronograph::nTimeSteps() + timeStep_; }

  private:
    unsigned int year_;
    unsigned int timeStep_;
};


#endif // Chronograph_H
