/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// Cohort.cpp 

#include "pragma.h"
#include "Cohort.h"
#include "Stock.h"
#include "Fishery.h"
#include "Chronograph.h"
#include "Geographer.h"
#include "HarvestManager.h"
#include "TransitionMatrix.h"
#include "NonNeg.h"
#include <stdio.h>


// ctor for CohortID
CohortID::CohortID(const Stock* stock, unsigned int broodYear, Age age,
               MaturationStatus maturity, MarkStatus mark,
               TagStatus tag, Sex sex, GrowthGroup growthGroup)
    : stock_(stock), broodYear_(broodYear), age_(age),
      run_(stock->run()), prodType_(stock->prodType()), 
      maturity_(maturity), mark_(mark), tag_(tag), sex_(sex),
      growthGroup_(growthGroup)
{}

// return a string representing this cohort id;  only need to calculate this once,
// then cache the result for speed.  

const RWCString& CohortID::idString() const
{
    if (idString_.isNull()) {
        char buf[64];
        sprintf(buf, "%s%4d%c%c%c%c%c%c%c", 
            stock_->get_abbrv(), broodYear_,
            (run_ == FALL) ? 'F' : 'S',
            (prodType_ == P_UNKNOWN) ? 'X' : ((prodType_ == WILD) ? 'W' : 'H'),
            (maturity_ == IMMATURE) ? 'I' : 'M',
            (mark_ == M_UNKNOWN) ? 'X' : ((mark_ == MARKED) ? 'M' : 'U'),
            (tag_ == T_UNKNOWN) ? 'X' : ((tag_ == TAGGED) ? 'T' : 'U'),
            (sex_ == S_UNKNOWN) ? 'X' : ((sex_ == MALE) ? 'M' : 'F'),
            (growthGroup_ == G_UNKNOWN) ? 'X' : ((growthGroup_ == G1) ? '1' : 
            ((growthGroup_==G2) ? '2' : ((growthGroup_==G3) ? '3' : 
            ((growthGroup_==G4) ? '4' : '5' ))))
            );
        // cast away const to store cache value
        CohortID* mythis = const_cast<CohortID*>(this);
        mythis->idString_ = buf;
    }
    return idString_;
}

void CohortID::
clearIdStringCache() const  // allow the cache to be cleared even if const
{
    CohortID* mythis = const_cast<CohortID*>(this);
    mythis->idString_.resize(0);
}

// preconditions: system monostates Geographer, Chronograph, and
// FisheryManager have already been configured, in order to provide
// proper ranges for the matrices and for HvMort object setup.

Cohort::Cohort(const CohortID& id) : id_(id)
{
    // resizes and zeroes
    currentAbd_.resize(Geographer::nRegions());
    naturalMortality_.resize(Geographer::nRegions());
}

// copy ctor makes a deep copy
Cohort::Cohort(const Cohort& rhs) : id_(rhs.id_)
{
    // the RW math vectors require the use of the assignment operator to get a deep copy
    // (copy ctor only makes a shallow copy)
    // resizes and zeroes
    currentAbd_.resize(Geographer::nRegions());
    naturalMortality_.resize(Geographer::nRegions());

    currentAbd_ = rhs.currentAbd_;
    naturalMortality_ = rhs.naturalMortality_;
}

Cohort::~Cohort()
{}

// merge the data from another cohort.  only performed if ids match.
void Cohort::
merge(const Cohort& otherCohort)
{
    if (id_ == otherCohort.id()) {
        currentAbd_ += otherCohort.currentAbd_;
        naturalMortality_ += otherCohort.naturalMortality_;
    }
}


bool Cohort::
cohortReady(int calendarYear) const
{
    // compute age
    int age = calendarYear - id_.broodYear_;
    
    // check if the cohort is too old - this shouldn't really ever happen
    if (age > id_.stock_->maxAge())
        return false;
    
    if (age >= 1)
        return true;
    else
        return false;
}


Age Cohort::
oceanAge() const
{
    if (id_.run_ == CohortID::SPRING) {
        Age age = id_.age_;
        --age;
        return age;
    }
    else
        return id_.age_;
}

// set the age for a cohort in the given year.  return false if the cohort is too young or old.

bool Cohort::setAge(int calendarYear)
{
    int age = calendarYear - id_.broodYear_;
    
    if (age < Age::bottom() || age > id_.stock_->maxAge())
        return false;
    else {
        // this is the lone exception to the CohortID being constant
        CohortID& id = const_cast<CohortID&>(id_);
        id.age_.assign_from_int(age);
    }
    return true;
}

void
Cohort::migrate(const TransitionMatrix& transMat, const Chronograph& clock)
{
    static AbundanceVector tmp(currentAbd_.length(), rwUninitialized);
    transMat.transition(currentAbd_, tmp);
    currentAbd_ = tmp;
}

Age
Cohort::firstHarvestAge() const
{
    return id_.stock_->firstHarvestAge();
}


