/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// CohortManager - repository and manager for all cohorts in the system

#include "CohortManager.h"
#include "CohortState.h"
#include "SystemClock.h"
#include "Stocks.h"
#include "Cohort.h"
#include "GenericArray.h"
#include "Stock.h"
#include "log.h"

#ifdef RWDEBUG
#include <assert.h>
#endif

DestructiveCohortList CohortManager::pendingCohorts_;
unsigned CohortManager::firstBroodYear_ = 1000000;  // arbitrarily large number

// config() - setup occuring before data is read
void
CohortManager::config()
{
    pendingCohorts_.clearAndDestroy();
    firstBroodYear_ = 1000000;
}

// post-simulation wrapup
void 
CohortManager::simulationWrapup()
{
    // delete any cohorts left over
    pendingCohorts_.clearAndDestroy();
}

Cohort*
CohortManager::newCohort(const CohortID& id)
{
    Cohort *cohort = new Cohort(id);
    // keep track of the earliest brood year
    if (id.broodYear() < firstBroodYear_)
        firstBroodYear_ = id.broodYear();
    return cohort;
}

// make and return a new cohort and put it on the pending list
Cohort* CohortManager::
newPendingCohort(const CohortID& id)
{
	Cohort* cohort = newCohort(id);
    pendingCohorts_.insert(cohort);
    return cohort;
}

// make a new cohort and activate it in the appropriate stock, returning a handle
Cohort* CohortManager::
newActiveCohort(const CohortID& id)
{
    Cohort* cohort = newCohort(id);
    Stock* stock = const_cast<Stock*>(id.stock()); // cast away const
    stock->activateCohort(auto_ptr<Cohort> (cohort));
    return cohort;
}

// make new age0 cohorts. the stock handles this.
void CohortManager::
makeNewAge0Cohorts(double abundance, unsigned region, Stock& stock, int broodYear)
{
    stock.newAge0Cohorts(abundance, region, broodYear);
}

// find the active mature cohort corresponding to the supplied cohort.  if
// not found, create a new one, and insert it into the master list and
// the "active" list maintained by the stock.  return the mature
// cohort.  precondition: passed in cohort is immature
Cohort*
CohortManager::findOrCreateMatureCohort(const Cohort* cohort)
{
#ifdef RWDEBUG    
    assert(cohort->immature());
#endif    
    
    // create the ID matching the cohort we want
    CohortID matId = cohort->id();
    matId.maturity(CohortID::MATURE);
    
    // search the "active cohorts" list, which is kept in each stock
    Stock* stock = const_cast<Stock*>(cohort->stock()); // cast away const
    Cohort* matureCohort = stock->findActiveCohort(matId);

    // if mature cohort not found, create one
    if (!matureCohort) {
        matureCohort = newCohort(matId);
        // activate and give ownership to the stock
        stock->activateCohort(auto_ptr<Cohort>(matureCohort));
    }
    // return a handle
    return matureCohort;
}


// activate all pending cohorts for the current system year.  visit each cohort
// on the pending list, and if it is "ready" (i.e. the age is in range) then remove 
// it from the pending list and add it to the active list (kept in the stocks)
void CohortManager::
activatePendingCohorts()
{
    int year = ::SystemClock->currentCalendarYear();
    CohortIter next(pendingCohorts_);
    Cohort* cohort;
    while (cohort = next()) {
        if (cohort->cohortReady(year)) {
            // need non-const stock ptr
            int s = cohort->stock()->index();
            // give ownership to the stock
            ::Stocks[s].activateCohort(auto_ptr<Cohort>(cohort));
            next.remove();
        }
    }
}

// find a particular cohort
void
CohortManager::findPendingCohorts(const Stock* stock, unsigned broodyear, CohortList& list) 
{
    CohortIter next(pendingCohorts_);
    Cohort* cohort;
    while (cohort = next()) {
        if (cohort->stock() == stock && cohort->broodYear() == broodyear) {
            list.append(cohort);
        }
    }
}

// find a currently active cohort
Cohort* CohortManager::
findActiveCohort(const CohortID& id)
{
    // the active cohort list is currently kept in the stocks
    Stock* stock = const_cast<Stock*>(id.stock());
    return (stock->findActiveCohort(id));
}

// age active cohorts.  also activate pending cohorts if this is the start of a new year
void CohortManager::
ageCohorts() 
{
    // check to see if this is the start of a year
    if (::SystemClock->currentTimeIndex() == 0) {
        activatePendingCohorts();

        // we could do aging every timestep, but for now we don't
        // the active cohort lists are kept in the stocks
        for (int i=0; i < ::Stocks.num(); ++i) {
            ::Stocks[i].ageCohorts();
        }
    }
}

// return an index suitable for array referencing corresponding to the given 
// calendar-based brood year
unsigned 
CohortManager::broodYearIndex(unsigned broodYear)
{
    unsigned index = broodYear - firstBroodYear_;
    if (index < 0)
        ::LogMsg(L_Fatal, "bad brood year index requested");
    return index;
}

void CohortManager::
saveState(CohortState& cohortState)
{
    // clear any residual state
    cohortState.pendingCohorts_.clearAndDestroy();
    cohortState.activeCohorts_.clearAndDestroy();

    // first make a deep copy of the pending cohort list, using heap memory
    CohortIter next(pendingCohorts_);
    Cohort* cohort;
    while (cohort = next()) {
        cohortState.pendingCohorts_.insert(new Cohort(*cohort));
    }

    // then make deep copies of all the active cohort lists kept in the stocks
    for (int i=0; i<::Stocks.num(); ++i) {
        DestructiveCohortList& targetList = *(new DestructiveCohortList);
        const CohortList& sourceList = ::Stocks[i].getCohorts();
        CohortList::const_iterator ptr = sourceList.begin();
        while (ptr != sourceList.end()) {
            targetList.insert(new Cohort(**ptr));
            ++ptr;
        }
        cohortState.activeCohorts_.append(&targetList);
    }
}

void CohortManager::
restoreState(const CohortState& cohortState)
{
    // clear the pending cohort list and make deep copy of the one from the saved state
    pendingCohorts_.clearAndDestroy();

    DestructiveCohortList::const_iterator ptr = cohortState.pendingCohorts_.begin();
    while (ptr != cohortState.pendingCohorts_.end()) {
        pendingCohorts_.insert(new Cohort(**ptr));
        ++ptr;
    }

    // clear the active cohort lists in the stocks and make deep copies from the state
    for (int i=0; i<::Stocks.num(); ++i) {
        ::Stocks[i].destroyCohorts();

        const DestructiveCohortList& sourceList = *(cohortState.activeCohorts_(i));
        DestructiveCohortList::const_iterator ptr = sourceList.begin();
        while (ptr != sourceList.end()) {
            ::Stocks[i].activateCohort(auto_ptr<Cohort>(new Cohort(**ptr)));
            ++ptr;
        }
    }
}