/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// Cohort Manager monostate

#ifndef CohortManager_H
#define CohortManager_H

#include "CohortList.h"
#include "Age.h"

class Stock;
class GenericArrayIndex;
class CohortState;

class CohortManager {
  public:
    CohortManager() {}
    ~CohortManager() {}
    
    static void config();
    static void simulationWrapup();

    //
    // factory methods for cohorts
    //
    // make a new cohort and keep it on the pending list
    static Cohort* newPendingCohort(const CohortID&);

    // make a new active cohort
    static Cohort* newActiveCohort(const CohortID&);

    // find an active mature cohort matching the supplied one, or create one.
    // the active list in the stock will be updated if a new cohort
    // was created.
    static Cohort* findOrCreateMatureCohort(const Cohort* cohort);

    // make new age0 cohorts. 
    static void makeNewAge0Cohorts(double abundance, unsigned region, Stock& stock, int broodYear);

    // find pending cohorts for the given stock and broodyear and insert into the supplied list.
    static void findPendingCohorts(const Stock* stock, unsigned broodyear, CohortList& list);

    // find a currently active cohort
    static Cohort* findActiveCohort(const CohortID&);

    // activate pending cohorts at the start of a new year, and age all active cohorts
    // this routine can be called at every timestep
    static void ageCohorts();

    // return an index suitable for array referencing corresponding to the given 
    // calendar-based brood year
    static unsigned broodYearIndex(unsigned broodYear);

    static void saveState(CohortState& cohortState);
    static void restoreState(const CohortState& cohortState);

  private:
    // create a new cohort
    static Cohort* newCohort(const CohortID&);

    // traverse the pending cohort list and activate all those who are 
    // ocean ready in the current year.
    static void activatePendingCohorts();
    
    // list of cohorts which have been created but not yet activated
    static DestructiveCohortList pendingCohorts_;

    static unsigned firstBroodYear_;  // earliest brood year of any cohort in the system
};

#endif // CohortManager_H
