/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "CohortParser.h"
#include "CohortTokens.h"
#include "Tokens.h"

#include "globals.h"
#include "CohortManager.h"


// ctor utility fn

void CohortParser::initKeymap()
{
    keymap_.insert(CohortTokens::WILD, wild);
    keymap_.insert(CohortTokens::HATCHERY, hatchery);
    keymap_.insert(CohortTokens::IMMATURE, immature);
    keymap_.insert(CohortTokens::MATURE, mature);
    keymap_.insert(CohortTokens::MARKED, marked);
    keymap_.insert(CohortTokens::UNMARKED, unmarked);
    keymap_.insert(CohortTokens::TAGGED, tagged);
    keymap_.insert(CohortTokens::UNTAGGED, untagged);
    keymap_.insert(CohortTokens::MALE, male);
    keymap_.insert(CohortTokens::FEMALE, female);
    keymap_.insert(CohortTokens::G1, g1);
    keymap_.insert(CohortTokens::G2, g2);
    keymap_.insert(CohortTokens::G3, g3);
    keymap_.insert(CohortTokens::G4, g4);
    keymap_.insert(CohortTokens::G5, g5);
    keymap_.insert(Tokens::END1, parseEndKeyword);
    keymap_.insert(Tokens::END2, parseEndKeyword);
}

CohortParser::CohortParser(Parser& p, const Stock* stock, RWCString& stockName)
    : SubParser(p), stockName_(stockName), cohortID_(stock, 0), abundance_(0),
      regionIndex_(0)
{
    initKeymap();

    // read the rest of the "Cohort" line, catching errors so as to continue
    int broodyear = 0;
    try {
        parseVar(broodyear);
        parseVar(abundance_);
        parseVar(regionIndex_);
    }
    catch (ParseError& err) {
        logError(err.errorMessage());
        logError("discarding cohort.\n");
        // skip the rest of the line
        nextDataLine();
        // valid stock is the flag for a valid cohort
        cohortID_.stock_ = 0;
    }
    if (cohortID_.stock_ != 0) {
        cohortID_.broodYear_ = broodyear;

        // convert user region number to internal index
        --regionIndex_;
    }
}

RWCString
CohortParser::context() const
{
    return("Stock " + stockName_ + " " + CohortTokens::COHORT);
}

void CohortParser::parseEndKeyword()
{
    if (cohortID_.stock_ != 0) {
        // create and insert new cohort into system
        Cohort* cohort = CohortManager::newPendingCohort(cohortID_);

        // set initial abundance
        cohort->abundance(regionIndex_) = abundance_;
    }
    SubParser::parseEndKeyword();
}

void CohortParser::wild()
{  cohortID_.prodType_ = CohortID::WILD; }

void CohortParser::hatchery()
{  cohortID_.prodType_ = CohortID::HATCHERY; }

// immature is default, but offered for completeness
void CohortParser::immature()
{  cohortID_.maturity_ = CohortID::IMMATURE; }

void CohortParser::mature()
{  cohortID_.maturity_ = CohortID::MATURE; }

void CohortParser::marked()
{  cohortID_.mark_ = CohortID::MARKED; }

void CohortParser::unmarked()
{  cohortID_.mark_ = CohortID::UNMARKED; }

void CohortParser::tagged()
{  cohortID_.tag_ = CohortID::TAGGED; }

void CohortParser::untagged()
{  cohortID_.tag_ = CohortID::UNTAGGED; }

void CohortParser::male()
{  cohortID_.sex_ = CohortID::MALE; }

void CohortParser::female()
{  cohortID_.sex_ = CohortID::FEMALE; }

void CohortParser::g1()
{  cohortID_.growthGroup_ = CohortID::G1; }

void CohortParser::g2()
{  cohortID_.growthGroup_ = CohortID::G2; }

void CohortParser::g3()
{  cohortID_.growthGroup_ = CohortID::G3; }

void CohortParser::g4()
{  cohortID_.growthGroup_ = CohortID::G4; }

void CohortParser::g5()
{  cohortID_.growthGroup_ = CohortID::G5; }
