/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "ConfigParser.h"
#include "ConfigTokens.h"
#include "TopTokens.h"
#include "Tokens.h"

#include "FisheryConfigParser.h"
#include "StockConfigParser.h"
#include "Chronograph.h"
#include "Geographer.h"
#include "globals.h"
#include "ErrorLogObject.h"

// ctor utility fn

void ConfigParser::initKeymap()
{
    keymap_.insert(Tokens::INCLUDE, parseInclude);
    keymap_.insert(ConfigTokens::STARTYEAR, startYear);
    keymap_.insert(ConfigTokens::ENDYEAR, endYear);
    keymap_.insert(ConfigTokens::NTIMESTEPS, nTimesteps);
    keymap_.insert(ConfigTokens::NREGIONS, nRegions);
    keymap_.insert(ConfigTokens::FISHERYNAME, fisheryName);
    keymap_.insert(ConfigTokens::STOCKNAME, stockName);
    keymap_.insert(Tokens::END1, parseEndKeyword);
    keymap_.insert(Tokens::END2, parseEndKeyword);
}

ConfigParser::ConfigParser(const RWCString& fileName)
 : SubParser(fileName, &::errorLogObject), startYear_(-1)
{
    initKeymap();

    // when parsing an include file, all previous fishery and stock configs
    // are PRESERVED
}

ConfigParser::ConfigParser(Parser& p) : SubParser(p), startYear_(-1)
{
    initKeymap();

    // when parsing new config information, all previous fishery and stock configs
    // are discarded
    Fisheries.clearAndDestroy();
    Stocks.clearAndDestroy();
}

RWCString
ConfigParser::context() const
{
    return(TopTokens::CONFIGURATION);
}

void
ConfigParser::parseInclude()
{
    
    RWCString filename = nextToken();
    try {
        ConfigParser p(filename);
        p.parse();
    }
    catch(ParseError& err) {
        // modify the error message if it was BadFile (add current
        // filename and line number)
        if (err == ParseError::BadFile)
            throw pError(ParseError::BadFile,
            "unable to open file \"" + filename + "\""
            );
        else throw;
    }
}

// stash the start year
void ConfigParser::startYear()
{
    parseVar(startYear_);
}

// parse the end year and configure the global Chronographer
void ConfigParser::endYear()
{
    int endYear;
    parseVar(endYear);

    if (startYear_ == -1 || endYear < startYear_) {
        if (startYear_ == -1) {   
            throw pError(ParseError::BadData, "startYear must come before endYear");
        }
        else {
            char buf[1024];
            ostrstream s(buf, 1024);
            s << "Unable to configure startYear = " << startYear_
                << " endYear = " << endYear;
            buf[s.pcount()] = 0;
            RWCString msg(buf);
            throw pError(ParseError::BadData, msg);
        }
    }

    if (!::SystemClock) 
        ::SystemClock = new Chronograph();
    if (!::SystemClock)
        throw; // alloc error
    ::SystemClock->configYears((endYear - startYear_) + 1, startYear_);
}

void
ConfigParser::nTimesteps()
{
    int nsteps;
    parseVar(nsteps);
    if (!::SystemClock) 
        ::SystemClock = new Chronograph();
    if (!::SystemClock)
        throw; // alloc error
    ::SystemClock->configTimeSteps(nsteps);
}

void ConfigParser::nRegions()
{
    int nregions;
    parseVar(nregions);
    Geographer::config(nregions);
}

void ConfigParser::fisheryName()
{
    RWCString fname;
    parseVar(fname);
    FisheryConfigParser fisheryconfigp(*this, fname);
    fisheryconfigp.parse();
}

void ConfigParser::stockName()
{
    RWCString sname;
    parseVar(sname);
    StockConfigParser stockconfigp(*this, sname);
    stockconfigp.parse();
}


