/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// fishery class and related declarations

#ifndef FISHERY_H
#define FISHERY_H

#include "Bool.h"
#include "RegionList.h"
#include "StockList.h"
#include "Array3D.h"

#include <rw/tpordvec.h>
#include <rw/cstring.h>

class Cohort;
class IOCrud;
class Stock;
class HvMort;
class FisheryUnit;

typedef RWTPtrOrderedVector<FisheryUnit> FisheryUnitList;

class Fishery {
    
  public:
    Fishery(const RWCString& nm);
    ~Fishery();

    // inits
    void config();
    void activate(int year, int time, int region);
    void clearSchedule();

    // data access
    const char* get_name() const { return name.isNull() ? "" : name; }
    const char* get_abbrv() const { return abbrv.isNull() ? "" : abbrv; }
    unsigned index() const { return index_; }
    unsigned number() const { return number_; }

    const FisheryUnitList& fisheryUnits() const { return fisheryUnits_; }
    
    double subLegalCnrMortRate(const Region&) const;
    double dropOffMortRate(const Region&) const;

    const RWTPtrOrderedVector<HvMort>& harvests() const { return harvests_; }

    // intermediate results data access

    // relative harvest rate is policy scalar, capped at 1.0, for CNRs
    double relHr(const Region& region) const;
    double encounterRate() const { return encounterRate_; }	// current encounter rt
    void encounterRate(double rate) { encounterRate_ = rate; }  // current encounter rt
      
    // message methods
    void makeHarvestList();
    void takeHarvests();
    void setShakers();
    void setCNR();

    void applyHarvestMorts();
    
    // results
    double legalCatch() const; // current timestep total catch
    double legalCatch(const Region& region) const; // current catch by region
    
    // operators (for Rogue Wave)
    int operator==(const Fishery &rhs) const
	{ return (&rhs==this); }
    
private:
    friend class FisheryConfigParser;

    void index(unsigned index) { index_ = index; } // this MUST be set properly at config time
    void clearFisheryUnits();   // clear and empty the contents of the current FisheryUnitList
    void setDefaultSchedule();  // set fishery schedule to default; always on

    const RWCString name;
    const RWCString abbrv;
    unsigned index_;			// unique ID suitable for array indexing, set at config
    unsigned long number_;      // externally assigned number

    // mortality data for the current timestep.   
    // copies are kept here and in the FisheryUnits.  the FisheryUnits own the pointers
    RWTPtrOrderedVector<HvMort> harvests_; 

    // gear types
    enum GearType { TROLL, NET, SPORT }; 	
    GearType gear;		// gear type

    // cohort characterisics
    double encounterRate_;	// current encounter rate (computed)

    // list of fishery unit activity.  this uses objects from the fisheryUnitPool_.
    FisheryUnitList fisheryUnits_;          
    Array3D<bool> activitySchedule_;        // yearXtimestepXregion of fishery activity

    // memory pool for fisheryUnits_
    FisheryUnitList fisheryUnitPool_;
};
    
#endif
