/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "Fishery.h"
#include "FisheryUnit.h"
#include "FisheryPolicy.h"
#include "HvMort.h"
#include "HarvestManager.h"
#include "FisheryManager.h"

// time independent harvesting.  this is the main entry point to 
// have the fisheryUnit perform its harvest, typically called from
// the owning Fishery.  the strategy is to locate the propery 
// FisheryPolicy controlling this FisheryUnit, and to pass control
// to that object.  the FisheryPolicy will at some point invoke the 
// harvest(effortScalar) method to actually cause harvesting in each
// HvMort object.

void FisheryUnit::harvest()
{
    // find the right FisheryPolicy from the HarvestManager
    FisheryPolicy* policy = FisheryManager::findPolicy(this);
    // cede control to the policy for harvesting
    policy->harvest(this);
}

// helper fn to call the harvest() fn in HvMort class
static void hvmHarvest(HvMort* hvm, void *data)
{
    double effortScalar = *((double *) data);
    hvm->harvest(effortScalar);
}

void FisheryUnit::harvest(double effortScalar)
{
    harvests_.apply(hvmHarvest, (void *) &effortScalar);
}


void FisheryUnit::makeHarvestList()
{
    harvests_.clearAndDestroy();
    HarvestManager::createHarvestMorts(this, harvests_);
}

// add all the FisheryUnit HvMort pointers to the given list
void FisheryUnit::getHarvests(RWTPtrOrderedVector<HvMort>& harvests) const
{
    for (int i=0; i < harvests_.entries(); ++i) {
        harvests.append(harvests_(i));
    }
}

// total legal catch of all harvests
double FisheryUnit::legalCatch() const {
    double total = 0;
    for (int i=0; i < harvests_.entries(); ++i) {
        total += harvests_(i)->legalCatch();
    }
    return total;
}

