/**********************************************************************
  Columbia Basin Research Software Library Tools
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

#ifndef FreeFormReader_H
#define FreeFormReader_H

// FreeFormReader class.  Opens a buffer and provides fns for reading
// tokens from minimally formatted files.  Uses a LineBuffer object to
// manage the buffer.  Primary interface is provision of nextToken()
// fn which skips comments, blank lines, and line boundaries.  When a
// null token is returned, the end of the file has been reached.
// nextDataLine() is also provided to skip to the beginning of the
// next line containing data.  peekToken() returns a peek at the next token 
// to be returned by nextToken(), without advancing beyond it. 

// Comment recognition is handled by a
// contained CommentRecognizer object.  Any comment recognized is
// assumed to extend to the end of the line.  The FreeFormReader class
// may throw ParseError exceptions (via the LineBuffer).

// Objects of this class may be copied using the standard copy
// constructor.  The LineBuffer resource will be shared by any copies.
// A member class which holds the LineBuffer as a reference-counted
// object and a corresponding smart pointer are used to manage the
// memory.  No copy on write is performed, hence the resource is fully
// shared.

#include "LineBuffer.h"
#include "RCObject.h"
#include "RCPtr.h"

// First declare the CommentRecognizer class.  This is a very simple
// helper class which provides one operation: recognition of a
// comment.  The interface is the overloaded fn call operator taking a
// token (RWCString) as a parameter.  Since we only have one type of
// comment, the class resides here.  If we need different types of
// comments in the future, then this should be placed in another file
// and turned into an abstract base class, with the actual comment
// types in derived classes.  In that case the CommentRecognizer for
// the FreeFormReader would most likely be passed in the ctor.
//
// This CommentRecognizer recognizes an empty token or the '#' symbol
// at the beginning of a token as heralding a comment.

class CommentRecognizer {
  public:
    RWBoolean operator() (const RWCString &token) const
    { return (token.isNull() || token(0) == '#') ? TRUE : FALSE; }
};

// FreeFormReader class

class FreeFormReader {
  public:
    FreeFormReader(const RWCString& fileName,
		      const char* separators=0, const char* operators=0);
    
    virtual ~FreeFormReader() {}
    
    const char* nextDataLine(); // skip rest of current line
    RWCString nextToken();      // advance file and return next token
    RWCString peekToken() const      // peek at next token
    { return tokenBuf_->nextToken_.value_; }

    RWCString lastToken() const      // return copy of last token returned
    { return tokenBuf_->currentToken_.value_; }

    long lineNumber() const     // return line number of last token returned (not from peek)
    { return tokenBuf_->currentToken_.lineNum_; } 

    RWCString fileName() const { return tokenBuf_->fileName(); }

  private:
    RWCString advanceToken();  // get the next token from the line buffer

    // reference counted class to handle a shared LineBuffer and a token lookahead
    struct SharedTokenBuffer : public LineBuffer, private RCObject {
        SharedTokenBuffer(const RWCString& filename, 
						  const char* separators=0,
						  const char* operators=0)
            : LineBuffer(filename, separators, operators) {}
        
        // copy ctor not defined; this shared resource may not be copied
        SharedTokenBuffer(const SharedTokenBuffer& rhs); 
        
        // class for internally tracking tokens and line numbers
        struct InternalToken {
            InternalToken() : lineNum_(0) {}
            
            RWCString value_;
            int lineNum_;
        };
        // keep copies of the current and lookahead tokens associated with the line buffer
        InternalToken currentToken_;    // last token returned
        InternalToken nextToken_;       // next token to return

        friend class RCPtr<SharedTokenBuffer>;
    };
    RCPtr<SharedTokenBuffer> tokenBuf_;
    CommentRecognizer isComment_;
	RWCString myOperators_;  // must persist while tokenizer is active,
	                         // since tokenizer will maintain a reference
	                         //  to the data of this string.
};

#endif // FreeFormReader_H
