/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include <assert.h>

#include "globals.h"
#include "GenArrayParser.h"
#include "Tokens.h"

// protected ctor for derived class to initialize base class
GenArrayParser::
GenArrayParser(Parser& p)
: SubParser(p)
{
    // pure virtual fns to perform initializations.  these actually
    // must be called in the derived class ctor, but are shown here
    // for demonstration purposes.
    
    // initState();
    // initKeymap();
}


RWCString 
GenArrayParser::context() const
{
    return(stateStack.top()->context);
}

// on end token, pop the state.  if there are still some left, flag
// the parser to continue.  note that the first state in the stack
// contains the id list which "owns" all the ids, and must free them
// if it is being popped.  since the state itself is a pointer, it too
// must be destroyed on every pop.

void
GenArrayParser::parseEnd()
{
    SubParser::parseEndKeyword();
    if (stateStack.entries() == 1) {
        State* oldstate = stateStack.pop();
        oldstate->ids.clearAndDestroy();
        delete oldstate;
    }
    else {
        delete(stateStack.pop());
        clearDone();
    }
}

unsigned int
GenArrayParser::State::operator==(const State&) const
{
    // this fn should never be called
    assert(0);
    return 0;
}

// various subtokens restrict the id list.  instead of calling a
// subparser, this is handled by pushing a new state.

// new year restricted id list
void
GenArrayParser::pushStateYears(const Range<int> years)
{ 
    char msg[256];
    ostrstream os(msg, 256);
    State* topState = stateStack.top();
    if (years.first == years.last)
        os << topState->context << " year " << years.first << '\0';
    else
        os << topState->context << " year range " << years.first << ':'
        << years.last << '\0';
    
    State* newstate = new State;
    newstate->context = msg;
    
    RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
    while (iter != (stateStack.top()->ids).end()) {
        GenericArrayIndex* id = *iter;
        unsigned calYear = id->yearIndex + Chronograph::baseYear();
        if (calYear >= years.first && calYear <= years.last)
            newstate->ids.insert(id);
        ++iter;
    }
    if (newstate->ids.isEmpty()) {
        logError(pError(ParseError::BadData,
            "warning, unable to find array entries for "
            + newstate->context)
            .errorMessage());
    }
    stateStack.push(newstate);
}

// new timestep restricted id list
void
GenArrayParser::pushStateTimeSteps(const Range<int> times)
{
    char msg[256];
    ostrstream os(msg, 256);
    
    State* topState = stateStack.top();
    if (times.first == times.last)
        os << topState->context << " timestep " << times.first << '\0';
    else
        os << topState->context << " timestep range " << times.first << ':'
        << times.last << '\0';
    
    State* newstate = new State;
    newstate->context = msg;
    
    RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
    while (iter != (stateStack.top()->ids).end()) {
        GenericArrayIndex* id = *iter;
        // this converts the internal representation of timesteps to the user notion 
        int timestep = id->timeIndex + 1;
        if (timestep >= times.first && timestep <= times.last)
            newstate->ids.insert(id);
        ++iter;
    }
    if (newstate->ids.isEmpty()) {
        logError(pError(ParseError::BadData,
            "warning, unable to find array entries for "
            + newstate->context)
            .errorMessage());
    }
    stateStack.push(newstate);
}

// new age restricted id list
void
GenArrayParser::pushStateAges(const Range<int> ages)
{
    char msg[256];
    ostrstream os(msg, 256);
    State* topState = stateStack.top();
    if (ages.first == ages.last)
        os << topState->context << " age " << ages.first << '\0';
    else
        os << topState->context << " age range " << ages.first << ':'
        << ages.last << '\0';
    
    State* newstate = new State;
    newstate->context = msg;
    
    RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
    while (iter != (stateStack.top()->ids).end()) {
        GenericArrayIndex* id = *iter;
        if (id->age >= ages.first && id->age <= ages.last)
            newstate->ids.insert(id);
        ++iter;
    }
    if (newstate->ids.isEmpty()) {
        logError(pError(ParseError::BadData,
            "warning, unable to find array entries for "
            + newstate->context)
            .errorMessage());
    }
    stateStack.push(newstate);
}

// new maturity restricted id list
void
GenArrayParser::pushStateMaturation(const Range<int> mats)
{
    char msg[256];
    ostrstream os(msg, 256);
    State* topState = stateStack.top();
    if (mats.first == mats.last)
        os << topState->context << " maturity " << mats.first << '\0';
    else
        os << topState->context << " maturity range " << mats.first << ':'
        << mats.last << '\0';
    
    State* newstate = new State;
    newstate->context = msg;
    
    RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
    while (iter != (stateStack.top()->ids).end()) {
        GenericArrayIndex* id = *iter;
        int matIndex = id->maturityIndex;
        if (matIndex >= mats.first && matIndex <= mats.last)
            newstate->ids.insert(id);
        ++iter;
    }
    if (newstate->ids.isEmpty()) {
        logError(pError(ParseError::BadData,
            "warning, unable to find array entries for "
            + newstate->context)
            .errorMessage());
    }
    stateStack.push(newstate);
}

// new mark status restricted id list
void
GenArrayParser::pushStateMark(const Range<int> marks)
{
    char msg[256];
    ostrstream os(msg, 256);
    State* topState = stateStack.top();
    if (marks.first == marks.last)
        os << topState->context << " mark status " << marks.first << '\0';
    else
        os << topState->context << " mark status range " << marks.first << ':'
        << marks.last << '\0';
    
    State* newstate = new State;
    newstate->context = msg;
    
    RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
    while (iter != (stateStack.top()->ids).end()) {
        GenericArrayIndex* id = *iter;
        int mark = id->mark;
        if (mark >= marks.first && mark <= marks.last)
            newstate->ids.insert(id);
        ++iter;
    }
    if (newstate->ids.isEmpty()) {
        logError(pError(ParseError::BadData,
            "warning, unable to find array entries for "
            + newstate->context)
            .errorMessage());
    }
    stateStack.push(newstate);
}

// new region restricted id list
void
GenArrayParser::pushStateRegions(const Range<int> locs)
{
    char msg[256];
    ostrstream os(msg, 256);
    State* topState = stateStack.top();
    if (locs.first == locs.last)
        os << topState->context << " region " << locs.first << '\0';
    else
        os << topState->context << " region range " << locs.first << ':'
        << locs.last << '\0';
    
    State* newstate = new State;
    newstate->context = msg;
    
    RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
    while (iter != (stateStack.top()->ids).end()) {
        GenericArrayIndex* id = *iter;
        // this converts the internal representation of regions to the user notion 
        int locIndex = id->regionIndex + 1;
        if (locIndex >= locs.first && locIndex <= locs.last)
            newstate->ids.insert(id);
        ++iter;
    }
    if (newstate->ids.isEmpty()) {
        logError(pError(ParseError::BadData,
            "warning, unable to find array entries for "
            + newstate->context)
            .errorMessage());
    }
    stateStack.push(newstate);
}

// new stock restricted id list
void
GenArrayParser::pushStateStocks(const Range<int> stocks)
{
    char msg[256];
    ostrstream os(msg, 256);
    State* topState = stateStack.top();
    // convert index to Stock number
    if (stocks.first == stocks.last) {
        int firstNum = ::Stocks[stocks.first].number();
        os << topState->context << " stock " << firstNum << '\0';
    }
    else {
        int firstNum = ::Stocks[stocks.first].number();
        int lastNum = ::Stocks[stocks.last].number();
        os << topState->context << " stock range " << firstNum << ':' << lastNum << '\0';
    }
    
    State* newstate = new State;
    newstate->context = msg;
    
    RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
    while (iter != (stateStack.top()->ids).end()) {
        GenericArrayIndex* id = *iter;
        if (id->stockIndex >= stocks.first && id->stockIndex <= stocks.last)
            newstate->ids.insert(id);
        ++iter;
    }
    if (newstate->ids.isEmpty()) {
        logError(pError(ParseError::BadData,
            "warning, unable to find array entries for "
            + newstate->context)
            .errorMessage());
    }
    stateStack.push(newstate);
}

// new fishery restricted id list
void
GenArrayParser::pushStateFisheries(const Range<int> fisheries)
{
    char msg[256];
    ostrstream os(msg, 256);
    State* topState = stateStack.top();

    // get the fishery numbers associated with these indices for the context
    if (fisheries.first == fisheries.last) {
        int firstNum = ::Fisheries[fisheries.first].number();
        os << topState->context << " fishery " << firstNum << '\0';
    }
    else {
        int firstNum = ::Fisheries[fisheries.first].number();
        int lastNum = ::Fisheries[fisheries.last].number();
        os << topState->context << " fishery range " << firstNum << ':' << lastNum << '\0';
    }
    
    State* newstate = new State;
    newstate->context = msg;
    
    RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
    while (iter != (stateStack.top()->ids).end()) {
        GenericArrayIndex* id = *iter;
        if (id->fisheryIndex >= fisheries.first && id->fisheryIndex <= fisheries.last)
            newstate->ids.insert(id);
        ++iter;
    }
    if (newstate->ids.isEmpty()) {
        logError(pError(ParseError::BadData,
            "warning, unable to find array entries for "
            + newstate->context)
            .errorMessage());
    }
    stateStack.push(newstate);
}

// new run type restricted id list
void
GenArrayParser::pushStateRuns(const Range<int> runs)
{
    char msg[256];
    ostrstream os(msg, 256);
    State* topState = stateStack.top();
    if (runs.first == runs.last)
        os << topState->context << " run type " << runs.first << '\0';
    else
        os << topState->context << " run type range " << runs.first << ':'
        << runs.last << '\0';
    
    State* newstate = new State;
    newstate->context = msg;
    
    RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
    while (iter != (stateStack.top()->ids).end()) {
        GenericArrayIndex* id = *iter;
        if (id->run >= runs.first && id->run <= runs.last)
            newstate->ids.insert(id);
        ++iter;
    }
    if (newstate->ids.isEmpty()) {
        logError(pError(ParseError::BadData,
            "warning, unable to find array entries for "
            + newstate->context)
            .errorMessage());
    }
    stateStack.push(newstate);
}

// parse fns for particular keywords
void
GenArrayParser::year()
{
    int year;
    parseVar(year);
    Range<int> years(year, year);
    pushStateYears(years);
}

void
GenArrayParser::years()
{
    Range<int> years;
    parseVar(years);
    pushStateYears(years);
}

void
GenArrayParser::time()
{
    int time;
    parseVar(time);
    Range<int> times(time, time);
    pushStateTimeSteps(times);
}

void
GenArrayParser::times()
{
    Range<int> times;
    parseVar(times);
    pushStateTimeSteps(times);
}

void
GenArrayParser::age()
{
    int age;
    parseVar(age);
    Range<int> ages(age, age);
    pushStateAges(ages);
}

void
GenArrayParser::ages()
{
    Range<int> ages;
    parseVar(ages);
    pushStateAges(ages);
}

void
GenArrayParser::region()
{
    int region;
    parseVar(region);
    Range<int> regions(region, region);
    pushStateRegions(regions);
}

void
GenArrayParser::regions()
{
    Range<int> regions;
    parseVar(regions);
    pushStateRegions(regions);
}

void
GenArrayParser::stock()
{
    // look ahead to see if a stock number instead of a name was given
    RWCString stockToken = peekToken();
    int num;
    istrstream s(const_cast<char *>(stockToken.data()));
    if (s >> num) {
        stockNum();
    }
    else {
        RWCString stockname;
        parseVar(stockname);
        
        Stock* stock = Stocks.find(stockname);
        if (stock) {
            Range<int> stocks(stock->index(), stock->index());
            pushStateStocks(stocks);
        }
        else {
            logError(pError(ParseError::BadData,
                "unable to find stock " + stockname + "; skipping...")
                .errorMessage());
            // push an empty state to process and discard data for this unknown stock
            State* newstate = new State;
            newstate->context = stateStack.top()->context + " unknown stock " + stockname;
            stateStack.push(newstate);    
        }	
    }
}

void GenArrayParser::
stockNum()
{
    int stocknum;
    parseVar(stocknum);
    // process by indices, not numbers
    const Stock* stock = ::findStockByNumber(::Stocks, stocknum);
    if (stock) {
        Range<int> stocks(stock->index(), stock->index());
        pushStateStocks(stocks);
    }
    else {
        logError(pError(ParseError::BadData,
            "unable to find stocknumber; skipping...")
            .errorMessage());
        // push an empty state to process and discard data for this unknown stock
        State* newstate = new State;
        newstate->context = stateStack.top()->context + " unknown stocknumber ";
        stateStack.push(newstate);    
    }
}

void
GenArrayParser::stocks()
{
    Range<int> stocks;
    parseVar(stocks);
    // parse by stock index, not number
    const Stock* firststock = ::findStockByNumber(::Stocks, stocks.first);
    const Stock* laststock = ::findStockByNumber(::Stocks, stocks.last);
    if (firststock && laststock) {
        stocks.first = firststock->index();
        stocks.last = laststock->index();
        pushStateStocks(stocks);
    }
    else {
        logError(pError(ParseError::BadData,
            "unable to identify stocknumber range; skipping...")
            .errorMessage());
        // push an empty state to process and discard data for this unknown stock
        State* newstate = new State;
        newstate->context = stateStack.top()->context + " unknown stock ";
        stateStack.push(newstate);    
    }
}


void
GenArrayParser::fishery()
{
    // look ahead to see if a fishery number instead of a name was given
    RWCString fisheryToken = peekToken();
    int num;
    istrstream s(const_cast<char *>(fisheryToken.data()));
    if (s >> num) {
        fisheryNum();
    }
    else {
        RWCString fisheryname;
        parseVar(fisheryname);
        
        Fishery* fishery = Fisheries.find(fisheryname);
        if (fishery) {
            Range<int> fisheries(fishery->index(), fishery->index());
            pushStateFisheries(fisheries);
        }
        else {
            logError(pError(ParseError::BadData,
                "unable to find fishery " + fisheryname + "; skipping...")
                .errorMessage());
            // push an empty state to process and discard data for this unknown fishery
            State* newstate = new State;
            newstate->context = stateStack.top()->context + " unknown fishery " + fisheryname;
            stateStack.push(newstate);    
        }	
    }
}

void GenArrayParser::
fisheryNum()
{
    int fisherynum;
    parseVar(fisherynum);
    // process by indices, not numbers
    const Fishery* fishery = ::findFisheryByNumber(::Fisheries, fisherynum);
    if (fishery) {
        Range<int> fisheries(fishery->index(), fishery->index());
        pushStateFisheries(fisheries);
    }
    else {
        logError(pError(ParseError::BadData,
            "unable to find fisherynumber; skipping...")
            .errorMessage());
        // push an empty state to process and discard data for this unknown Fishery
        State* newstate = new State;
        newstate->context = stateStack.top()->context + " unknown fisherynumber ";
        stateStack.push(newstate);    
    }
}

void
GenArrayParser::fisheries()
{
    Range<int> fisheries;
    parseVar(fisheries);
    // parse by fishery index, not number
    const Fishery* firstFishery = ::findFisheryByNumber(::Fisheries, fisheries.first);
    const Fishery* lastFishery = ::findFisheryByNumber(::Fisheries, fisheries.last);
    if (firstFishery && lastFishery) {
        fisheries.first = firstFishery->index();
        fisheries.last = lastFishery->index();
        pushStateFisheries(fisheries);
    }
    else {
        logError(pError(ParseError::BadData,
            "unable to identify fisherynumber range; skipping...")
            .errorMessage());
        // push an empty state to process and discard data for this unknown Fishery
        State* newstate = new State;
        newstate->context = stateStack.top()->context + " unknown fishery ";
        stateStack.push(newstate);    
    }
}

void
GenArrayParser::immature()
{
    pushStateMaturation(Range<int>(CohortID::IMMATURE, CohortID::IMMATURE));
}

void
GenArrayParser::mature()
{
    pushStateMaturation(Range<int>(CohortID::MATURE, CohortID::MATURE));
}


void
GenArrayParser::marked()
{
    pushStateMark(Range<int>(CohortID::MARKED, CohortID::MARKED));
}

void
GenArrayParser::unmarked()
{
    pushStateMark(Range<int>(CohortID::UNMARKED, CohortID::UNMARKED));
}

void
GenArrayParser::run()
{
    RWCString run;
    parseVar(run);
    run.toLower();
    if (run == CohortTokens::FALL)
        pushStateRuns(Range<int>(CohortID::FALL, CohortID::FALL));
    else if (run== CohortTokens::SPRING)
        pushStateRuns(Range<int>(CohortID::SPRING, CohortID::SPRING));
    else {
        // unrecognized run type
     	throw pError(ParseError::BadData, "unrecognized stock run type", run);
   }
}

