/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#ifndef GenArrayParser_H
#define GenArrayParser_H

#include "SubParser.h"
#include "KeywordProcessor.h"
#include "GenericArray.h"
#include "GenArrayTokens.h"
#include "Tokens.h"

#include <rw/tstack.h>
#include <rw/tvordvec.h>
//#include <rw/tpslist.h>
#include <rw/tpordvec.h>

/* The GenArrayParser handles GenericArray parameters.  It
 * maintains a list of GenericIDs which define the set of elements over
 * which to apply a given paramter value.  This list is part of the
 * "state" of this parser.  Certain tokens cause the parser to push a
 * new state on a state-stack, and also reduce the size of the
 * GenericID list in the new state.  For example, a "year" token would
 * push a new state where the ID list consists only of those IDs from
 * the previous list which match the defined year.  In this way the
 * input data file may specify arbitrarily specific sets of IDs over
 * which to apply a particular paramter value.
 */

class GenArrayParser : public SubParser {
  public:
    GenArrayParser(Parser& p);
    virtual ~GenArrayParser() {}
    
    // a fn to retreive the current "context", also for error reporting
    virtual RWCString context() const;

    // GenArray parsing fns
    void year();
    void years();
    void time();
    void times();
    void region();
    void regions();
    void stock();
    void stockNum();
    void stocks();
    void fishery();
    void fisheryNum();
    void fisheries();
    void age();
    void ages();
    void immature();
    void mature();
    void marked();
    void unmarked();
    void run();

    virtual void parseEnd();

    // this subparser works by keeping a list of the GenericIDs currently
    // under examination.
    class State {
    public:
        State() {}
        
//        RWTPtrSlist<GenericArrayIndex> ids;
        RWTPtrOrderedVector<GenericArrayIndex> ids;
        RWCString context;
        
        unsigned int operator==(const State&) const;
    };
    RWTStack<State*, RWTValOrderedVector<State*> > stateStack;
    
  private:
    // ctor utility fns, must be defined in derived classes and called
    // in derived class ctors
    virtual void initKeymap() = 0;
    virtual void initState() = 0;

    // fns which push new states
    void pushStateYears(const Range<int> years);
    void pushStateTimeSteps(const Range<int> times);
    void pushStateAges(const Range<int> ages);
    void pushStateMaturation(const Range<int> mats);
    void pushStateMark(const Range<int> marks);
    void pushStateStocks(const Range<int> stocks);
    void pushStateRegions(const Range<int> locs);
    void pushStateFisheries(const Range<int> fisheries);
    void pushStateRuns(const Range<int> runs);
};

// template fn to set up GenArray tokens for a derived class keyword processor
template<class T> void initGenArrayKeymap(KeywordProcessor<T>& keymap)
{
    keymap.insert(GenArrayTokens::YEAR, &T::year);
    keymap.insert(GenArrayTokens::YEARS, &T::years);
    keymap.insert(GenArrayTokens::TIME, &T::time);
    keymap.insert(GenArrayTokens::TIMES, &T::times);
    keymap.insert(GenArrayTokens::REGION, &T::region);
    keymap.insert(GenArrayTokens::REGIONS, &T::regions);
    keymap.insert(GenArrayTokens::STOCK, &T::stock);
    keymap.insert(GenArrayTokens::STOCKNUM, &T::stockNum);
    keymap.insert(GenArrayTokens::STOCKS, &T::stocks);
    keymap.insert(GenArrayTokens::FISHERY, &T::fishery);
    keymap.insert(GenArrayTokens::FISHERIES, &T::fisheries);
    keymap.insert(GenArrayTokens::AGE, &T::age);
    keymap.insert(GenArrayTokens::AGES, &T::ages);
    keymap.insert(GenArrayTokens::RUN, &T::run);
    keymap.insert(CohortTokens::IMMATURE, &T::immature);
    keymap.insert(CohortTokens::MATURE, &T::mature);
    keymap.insert(CohortTokens::MARKED, &T::marked);
    keymap.insert(CohortTokens::UNMARKED, &T::unmarked);
    keymap.insert(Tokens::END1, &T::parseEnd);
    keymap.insert(Tokens::END2, &T::parseEnd);
}

#endif // GenArrayParser_H
