/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// GenericArray.cpp

#include "GenericArray.h"
#include "globals.h"
#include "Chronograph.h"
#include "Age.h"
#include "Stock.h"
#include "CohortManager.h"
#include "Geographer.h"

GenericArrayIndex::GenericArrayIndex()
  : stockIndex(0),
  broodYear(0),
  run(0),
  prodType(0),
  maturityIndex(0),
  mark(0),
  tag(0),
  sex(0),
  growthGroup(0),
  age(0),
  fisheryIndex(0), regionIndex(0),
  yearIndex(0), timeIndex(0)
{}

GenericArrayIndex::GenericArrayIndex(const Cohort* c, unsigned fishery, unsigned region, 
                                     unsigned year, unsigned time)
  : stockIndex(c ? c->stock()->index() : 0),
  broodYear(c ? CohortManager::broodYearIndex(c->broodYear()) : 0),
  run(c ? c->run() : 0),
  prodType(c ? c->prodType() : 0),
  maturityIndex(c ? c->maturity() : 0),
  mark(c ? c->markStatus() : 0),
  tag(c ? c->tagStatus() : 0),
  sex(c ? c->sex() : 0),
  growthGroup(c ? c->growthGroup() : 0),
  age(c ? (unsigned) c->age() : 0),
  fisheryIndex(fishery), regionIndex(region),
  yearIndex(year), timeIndex(time)
{}

GenericArrayIndex::GenericArrayIndex(const Cohort*c, unsigned fishery, unsigned region, 
                                     const Chronograph& clock)
  : stockIndex(c? c->stock()->index(): 0),
  broodYear(c ? CohortManager::broodYearIndex(c->broodYear()) : 0),
  run(c ? c->run() : 0),
  prodType(c ? c->prodType() : 0),
  maturityIndex(c ? c->maturity() : 0),
  mark(c ? c->markStatus() : 0),
  tag(c ? c->tagStatus() : 0),
  sex(c ? c->sex() : 0),
  growthGroup(c ? c->growthGroup() : 0),
  age(c ? (unsigned) c->age() : 0),
  fisheryIndex(fishery), regionIndex(region),
  yearIndex(clock.currentYearIndex()), timeIndex(clock.currentTimeIndex())
{}

GenericArrayIndex::GenericArrayIndex(const Chronograph& clock) 
  : stockIndex(0), broodYear(0), run(0), prodType(0), maturityIndex(0), 
  mark(0), tag(0), sex(0), growthGroup(0), age(0),
  fisheryIndex(0), regionIndex(0),
  yearIndex(clock.currentYearIndex()),
  timeIndex(clock.currentTimeIndex())
{}

GenericArrayIndex::GenericArrayIndex(unsigned year, unsigned time) 
  : stockIndex(0), broodYear(0), run(0), prodType(0), maturityIndex(0), 
  mark(0), tag(0), sex(0), growthGroup(0), age(0),
  fisheryIndex(0), regionIndex(0), yearIndex(year), timeIndex(time)
{}

int GenericArrayIndex::operator==(const GenericArrayIndex& rhs) const
{ 
    return (stockIndex == rhs.stockIndex
        && broodYear == rhs.broodYear
        && run == rhs.run
        && prodType == rhs.prodType
        && maturityIndex == rhs.maturityIndex
        && mark == rhs.mark
        && tag == rhs.tag
        && sex == rhs.sex
        && growthGroup == rhs.growthGroup
        && age == rhs.age
        && regionIndex == rhs.regionIndex
        && fisheryIndex == rhs.fisheryIndex
        && yearIndex == rhs.yearIndex
        && timeIndex == rhs.timeIndex);
}
    
// set the GenericArrayIndex to contain the size values for all the fields
void GenericArrayIndex::resetToSizes()
{
    stockIndex = Stocks.num();
    // number of possible brood years is 1 + the index of the last year, as computed by
    // the CohortManager
    broodYear = 1 + CohortManager::broodYearIndex(Chronograph::baseYear() + Chronograph::nYears() - 1);
    run = CohortID::N_RUN_TYPES;
    prodType = CohortID::N_PRODUCTION_STATES;
    maturityIndex = CohortID::N_MATURATION_STATES;
    mark = CohortID::N_MARKS;
    tag = CohortID::N_TAGS;
    sex = CohortID::N_SEXES;
    growthGroup = CohortID::N_GROWTH_GROUPS;
    age = Age::top()+1;
    fisheryIndex = Fisheries.num();
    regionIndex = Geographer::nRegions();
    yearIndex = Chronograph::nYears();
    timeIndex = Chronograph::nTimeSteps();
}

