/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#ifdef DEBUG
#include <assert.h>
#endif //DEBUG

#include "log.h"
#include "GenericArrayFactory.h"
#include "TimeStepArray.h"
#include "StockAgeTimeArray.h"
#include "StockYearArray.h"
#include "StockYearTimeRegionArray.h"
#include "StockYearAgeTimeArray.h"
#include "StockYearAgeTimeMatureArray.h"
#include "StockYearAgeTimeRegionArray.h"
#include "FisheryArray.h"
#include "FisheryYearArray.h"
#include "FisheryRegionTimeStockAgeArray.h"
#include "FisheryRegionTimeStockAgeMarkArray.h"
#include "FisheryYearAgeArray.h"
#include "FisheryYearStockArray.h"
#include "FisheryYearStockAgeArray.h"
#include "FisheryYearRunAgeArray.h"
#include "RunAgeMaturityTimeArray.h"
#include "MaturityTimeArray.h"
#include "GenArrayTokens.h"


// the type strings must be kept up to date with the available types
template <class T>
GenericArrayFactory<T>::GenericArrayFactory()
{
    GenericArrayTypeStrings[TimeStep] = GenArrayTokens::TIME;
    GenericArrayTypeStrings[StockAgeTime] = GenArrayTokens::STOCKAGETIME;
    GenericArrayTypeStrings[StockYear] = GenArrayTokens::STOCKYEAR;
    GenericArrayTypeStrings[StockYearTimeRegion] = GenArrayTokens::STOCKYEARTIMEREGION;
    GenericArrayTypeStrings[StockYearAgeTime] = GenArrayTokens::STOCKYEARAGETIME;
    GenericArrayTypeStrings[StockYearAgeTimeRegion] = GenArrayTokens::STOCKYEARAGETIMEREGION;
    GenericArrayTypeStrings[StockYearAgeTimeMaturity] = GenArrayTokens::STOCKYEARAGETIMEMATURITY;
    GenericArrayTypeStrings[FisheryRegionTimeStockAge] = GenArrayTokens::FISHERYREGIONTIMESTOCKAGE;
    GenericArrayTypeStrings[FisheryRegionTimeStockAgeMark] = GenArrayTokens::FISHERYREGIONTIMESTOCKAGEMARK;
    GenericArrayTypeStrings[FisheryYearAge] = GenArrayTokens::FISHERYYEARAGE;
    GenericArrayTypeStrings[FisheryYearStock] = GenArrayTokens::FISHERYYEARSTOCK;
    GenericArrayTypeStrings[FisheryYearStockAge] = GenArrayTokens::FISHERYYEARSTOCKAGE;
    GenericArrayTypeStrings[FisheryYear] = GenArrayTokens::FISHERYYEAR;
    GenericArrayTypeStrings[FisheryYearRunAge] = GenArrayTokens::FISHERYYEARRUNAGE;
    GenericArrayTypeStrings[Fishery] = GenArrayTokens::XFISHERY;
    GenericArrayTypeStrings[RunAgeMaturityTime] = GenArrayTokens::RUNAGEMATURITYTIME;
    GenericArrayTypeStrings[MaturityTime] = GenArrayTokens::MATURITYTIME;

#ifdef DEBUG
    // make sure all the types are accounted for
    assert(MaturityTime == 16 && MaturityTime == NTypes-1);
#endif // DEBUG    
}

// return the type code for the given string; return NTypes if not found
template <class T>
GenericArrayFactory<T>::GenericArrayType GenericArrayFactory<T>::
genericArrayType(const RWCString typestring) const
{
    for (int i=0; i < NTypes; ++i) {
	if (typestring == GenericArrayTypeStrings[i])
	    return GenericArrayType(i);
    }
    return NTypes;
}

// create a GenericArray of a specified dynamic type.  caller gets
// ownership.  index sizes are given in the GenericArrayIndex object.
template <class T>
GenericArray<T>* GenericArrayFactory<T>::
newGenericArrayFn2(GenericArrayFactory<T>::GenericArrayType t, const GenericArrayIndex& size)
{
    GenericArray<T>* array;
    switch (t) {
    case TimeStep:
        array = new TimeStepArray<T>(size.timeIndex);
        break;
        
    case StockAgeTime:
        array = new StockAgeTimeArray<T>
            (size.stockIndex,
            size.age,
            size.timeIndex);
        break;
        
    case StockYear:
        array = new StockYearArray<T>
            (size.stockIndex,
            size.yearIndex);
        break;
        
    case StockYearTimeRegion:
        array = new StockYearTimeRegionArray<T>
            (size.stockIndex,
            size.yearIndex,
            size.timeIndex,
            size.regionIndex);
        break;

    case StockYearAgeTime:
        array = new StockYearAgeTimeArray<T>
            (size.stockIndex,
            size.yearIndex,
            size.age,
            size.timeIndex);
        break;
        
    case StockYearAgeTimeMaturity:
        array = new StockYearAgeTimeMatureArray<T>
            (size.stockIndex,
            size.yearIndex,
            size.age,
            size.timeIndex,
            size.maturityIndex);
        break;
        
    case StockYearAgeTimeRegion:
        array = new StockYearAgeTimeRegionArray<T>
            (size.stockIndex,
            size.yearIndex,
            size.age,
            size.timeIndex,
            size.regionIndex);
        break;
        
    case FisheryRegionTimeStockAge:
        array = new FisheryRegionTimeStockAgeArray<T>
            (size.fisheryIndex,
            size.regionIndex,
            size.timeIndex,
            size.stockIndex,
            size.age);
        break;
        
    case FisheryRegionTimeStockAgeMark:
        array = new FisheryRegionTimeStockAgeMarkArray<T>
            (size.fisheryIndex,
            size.regionIndex,
            size.timeIndex,
            size.stockIndex,
            size.age,
            size.mark);
        break;
        
    case FisheryYearAge:
        array = new FisheryYearAgeArray<T>
            (size.fisheryIndex,
            size.yearIndex,
            size.age);
        break;
        
    case FisheryYearStock:
        array = new FisheryYearStockArray<T>
            (size.fisheryIndex,
            size.yearIndex,
            size.stockIndex);
        break;
        
    case FisheryYearStockAge:
        array = new FisheryYearStockAgeArray<T>
            (size.fisheryIndex,
            size.yearIndex,
            size.stockIndex,
            size.age);
        break;
        
    case FisheryYear:
        array = new FisheryYearArray<T>
            (size.fisheryIndex,
            size.yearIndex);
        break;
        
    case FisheryYearRunAge:
        array = new FisheryYearRunAgeArray<T>
            (size.fisheryIndex,
            size.yearIndex,
            size.run,
            size.age);
        break;
        
    case Fishery:
        array = new FisheryArray<T>(size.fisheryIndex);
        break;
        
    case RunAgeMaturityTime:
        array = new RunAgeMaturityTimeArray<T>
            (size.run,
            size.age,
            size.maturityIndex,
            size.timeIndex);
        break;
        
    case MaturityTime:
        array = new MaturityTimeArray<T>
            (size.maturityIndex,
            size.timeIndex);
        break;
        
    default:
        ::LogMsg(L_Fatal, "unrecognized generic array type in factory\n");
#ifdef DEBUG
        assert(0);
#endif	
    }
    return array;
}

// create a GenericArray of a specified dynamic type with a default
// value.  caller gets ownership.  index sizes are given in the
// GenericArrayIndex object.
template <class T>
GenericArray<T>* GenericArrayFactory<T>::
newGenericArrayFn3(GenericArrayFactory<T>::GenericArrayType t, const GenericArrayIndex& size, const T& dfltVal)
{
    GenericArray<T>* array;
    switch (t) {
    case TimeStep:
        array = new TimeStepArray<T>(size.timeIndex, dfltVal);
        break;
        
    case StockAgeTime:
        array = new StockAgeTimeArray<T>
            (size.stockIndex,
            size.age,
            size.timeIndex,
            dfltVal);
        break;
        
    case StockYear:
        array = new StockYearArray<T>
            (size.stockIndex,
            size.yearIndex,
            dfltVal);
        break;
        
    case StockYearTimeRegion:
        array = new StockYearTimeRegionArray<T>
            (size.stockIndex,
            size.yearIndex,
            size.timeIndex,
            size.regionIndex,
            dfltVal);
        break;

    case StockYearAgeTime:
        array = new StockYearAgeTimeArray<T>
            (size.stockIndex,
            size.yearIndex,
            size.age,
            size.timeIndex,
            dfltVal);
        break;
        
    case StockYearAgeTimeMaturity:
        array = new StockYearAgeTimeMatureArray<T>
            (size.stockIndex,
            size.yearIndex,
            size.age,
            size.timeIndex,
            size.maturityIndex,
            dfltVal);
        break;
        
    case StockYearAgeTimeRegion:
        array = new StockYearAgeTimeRegionArray<T>
            (size.stockIndex,
            size.yearIndex,
            size.age,
            size.timeIndex,
            size.regionIndex,
            dfltVal);
        break;
        
    case FisheryRegionTimeStockAge:
        array = new FisheryRegionTimeStockAgeArray<T>
            (size.fisheryIndex,
            size.regionIndex,
            size.timeIndex,
            size.stockIndex,
            size.age,
            dfltVal);
        break;
        
    case FisheryRegionTimeStockAgeMark:
        array = new FisheryRegionTimeStockAgeMarkArray<T>
            (size.fisheryIndex,
            size.regionIndex,
            size.timeIndex,
            size.stockIndex,
            size.age,
            size.mark,
            dfltVal);
        break;

    case FisheryYearAge:
        array = new FisheryYearAgeArray<T>
            (size.fisheryIndex,
            size.yearIndex,
            size.age,
            dfltVal);
        break;
        
    case FisheryYearStock:
        array = new FisheryYearStockArray<T>
            (size.fisheryIndex,
            size.yearIndex,
            size.stockIndex,
            dfltVal);
        break;
        
    case FisheryYearStockAge:
        array = new FisheryYearStockAgeArray<T>
            (size.fisheryIndex,
            size.yearIndex,
            size.stockIndex,
            size.age,
            dfltVal);
        break;
        
    case FisheryYear:
        array = new FisheryYearArray<T>
            (size.fisheryIndex,
            size.yearIndex,
            dfltVal);
        break;
        
    case FisheryYearRunAge:
        array = new FisheryYearRunAgeArray<T>
            (size.fisheryIndex,
            size.yearIndex,
            size.run,
            size.age,
            dfltVal);
        break;
        
    case Fishery:
        array = new FisheryArray<T>(size.fisheryIndex, dfltVal);
        break;
        
    case RunAgeMaturityTime:
        array = new RunAgeMaturityTimeArray<T>
            (size.run,
            size.age,
            size.maturityIndex,
            size.timeIndex,
            dfltVal);
        break;
        
    case MaturityTime:
        array = new MaturityTimeArray<T>
            (size.maturityIndex,
            size.timeIndex,
            dfltVal);
        break;
        
    default:
        ::LogMsg(L_Fatal, "unrecognized generic array type in factory\n");
#ifdef DEBUG
        assert(0);
#endif	
    }
    return array;
}

// create a new generic array with maximum indices
template <class T>
GenericArray<T>* GenericArrayFactory<T>::
newGenericArrayFn0(GenericArrayFactory<T>::GenericArrayType t)
{
    GenericArrayIndex size;
    size.resetToSizes();
    return newGenericArrayFn2(t, size);
}

// create a new generic array with maximum indices and a default value
template <class T>
GenericArray<T>* GenericArrayFactory<T>::
newGenericArrayFn1(GenericArrayFactory<T>::GenericArrayType t, const T& dfltVal)
{
    GenericArrayIndex size;
    size.resetToSizes();
    return newGenericArrayFn3(t, size, dfltVal);
}

