/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// HvMort.h - harvest mort repository

#ifndef HvMort_H
#define HvMort_H

#include "auto_ptr.h"
#include "Bool.h"
#include "HarvestProcess.h"

class Cohort;
class Fishery;
class Region;

// An HvMort object contains all the harvest data for a particular
// fishery/cohort/region triplet in a given region in the current
// timestep.  It is overwritten in subsequent timesteps.

class HvMort {
  public:
    HvMort(const Cohort& cohort, const Fishery& fishery, const Region& region, 
        auto_ptr<const HarvestProcess> process)
	: cohort_(cohort), fishery_(fishery), region_(region), harvestProcess_(process)
    { zero(); }

    ~HvMort()
    {}

    // message methods
    void applyHarvestMorts(); // update cohort abundances

    // mutators
    void zero();		// zero everything
    void harvest(double policyScalar);
    void setCNR();

    // accessors
    const Cohort& cohort() const { return cohort_; }
    const Fishery& fishery() const { return fishery_; }
    const Region& region() const { return region_; }

    double legalCatch() const { return legalCatch_; }
    double shakers() const { return shakers_; }
    double dropOffs() const { return dropOffs_; }
    double cnrSubLegal() const { return cnrSublegal_; }
    double cnrLegal() const { return cnrLegal_; }
    double cnrTotal() const { return cnrSublegal_ + cnrLegal_; }

    // the various shaker algorithms need these - it would be nice to 
    // come up with a better way to provide this functionality
    double& shakers() { return shakers_; }
    double& dropOffs() { return dropOffs_; }

    // cohort abundance corresponding to this harvest
    double cohortAbundance() const;
    
    // operators
    bool operator==(const HvMort& hvm) const
    { return (this == &hvm); }  // pointer identity
    
  private:
//    double hvRate() const;	// basic harvest rate

    const Cohort& cohort_;
    const Fishery& fishery_;
    const Region& region_;
    auto_ptr<const HarvestProcess> harvestProcess_; // configurable strategy for conducting harvest

    double legalCatch_;
    double shakers_;
    double dropOffs_;
    double cnrSublegal_;
    double cnrLegal_;
};
    
// external fn to call the zero() fn

//extern void zeroHarvest(HvMort* hvm, void *);

#endif // HvMort_H
