/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

// IterationManager.cpp

#include "IterationManager.h"
#include "SystemClock.h"


// static memory
IterationManager::ControlGroupDlist IterationManager::controlGroups_;
RWTPtrSlist<IterationControl> IterationManager::pendingRequests_;


// config just empties all the lists.  

void IterationManager::
config()
{ 
    pendingRequests_.clear();
    // this one owns the IterationControls, so they are destroyed here
    controlGroups_.clearAndDestroy();
}

// init by calling the init() method for every configured IterationControl
void IterationManager::
init()
{
    ControlGroupIter groupPtr(controlGroups_);
    while (++groupPtr) {
        RWTPtrSlistIterator<IterationControl> icPtr(groupPtr.key()->controls);
        while (++icPtr) {
            icPtr.key()->init();
        }
    }
}

// this method is called at the beginning of every timestep.  if there are pending iteration
// requests, it processes them and resets the state and clock, causing iteration to begin.  if there
// are no pending requests, it causes iteration controls to save or delete state as appropriate.

void IterationManager::beginTimeStep()
{
    if (!pendingRequests_.isEmpty()) {
        // all pending requests get to set control variables.  this should be done before restoring
        // state, in case the old state is required for calculating the new control variables.
        RWTPtrSlistIterator<IterationControl> request(pendingRequests_);
        while (++request) {
            request.key()->resetControlVariables();
        }
        // all requests have the same start step, so arbitrarily choose the first to 
        // reset the global SystemClock and restore the state.
        pendingRequests_.first()->restoreState();
        const TimeStamp& iterTime = pendingRequests_.first()->firstStep();
        ::SystemClock->resetClock(iterTime.yearIndex(), iterTime.timeIndex());

        pendingRequests_.clear();
    }
    else {
        // not starting a new iteration, so:
        //    save state in all IterationControls with the current time as startStep;
        //    delete state in groups where the latestLastStep has just passed
        TimeStamp current(*::SystemClock);
        int currentElapsed = current.elapsedTime();
        ControlGroupIter groupPtr(controlGroups_);
        while (++groupPtr) {
            if (groupPtr.key()->firstStep == current) {
                // make a SystemState snapshot, and hand a copy to each IterationControl
                // auto_ptr signifies the xfer of ownership
                const SystemState* state = new SystemState;
                RWTPtrSlistIterator<IterationControl> icPtr(groupPtr.key()->controls);
                while (++icPtr) {
                    // make copy of state with copy ctor and stuff it in an auto_ptr
                    auto_ptr<const SystemState> stateCopy(new SystemState(*state));
                    icPtr.key()->saveState(stateCopy);
                }
                delete state;
            }
            else if (groupPtr.key()->latestLastStep.elapsedTime() == currentElapsed-1) {
                RWTPtrSlistIterator<IterationControl> icPtr(groupPtr.key()->controls);
                while (++icPtr) {
                    icPtr.key()->deleteState();
                }
            }
        }
    }
}

// this method is called at the end of every timestep.  all IterationControls are 
// examined and allowed to request iteration.  requests are stored in the pendingRequests_
// list.  the startSteps of incoming requests are compared; older ones are discarded.

void IterationManager::endTimeStep()
{
    int latestStart = 0;
    ControlGroupIter groupPtr(controlGroups_);
    while (++groupPtr) {
        RWTPtrSlistIterator<IterationControl> icPtr(groupPtr.key()->controls);
        while (++icPtr) {
            IterationControl& ic = *icPtr.key();
            if (ic.iterationRequired()) {
                // if the first step of this request is the most recent yet seen,
                // discard old ones and start a new pending list
                if (ic.firstStep().elapsedTime() > latestStart) {
                    latestStart = ic.firstStep().elapsedTime();
                    pendingRequests_.clear();
                    pendingRequests_.insert(&ic);
                }
                else if (ic.firstStep().elapsedTime() == latestStart) {
                    // same first step as current list; append 
                    pendingRequests_.insert(&ic);
                }
                // any older first step requests are simply ignored
            }
        }
    }
}


// helper "find" fn to look for ControlGroups with a certain firstStep
bool IterationManager::
findFirstStepGroup(const IterationManager::ControlGroup* group, void* data)
{
    // the data is a ptr to the first step
    TimeStamp* timeStamp = static_cast<TimeStamp*> (data);
    if (group->firstStep == *timeStamp)
        return true;
    else 
        return false;
}

// list management.  entries are only inserted, or all deleted at once.  the structure of the 
// list is an outer list of ControlGroups, sorted by firstStep.  every IterationControl in a group
// has the same first step.  within a group, there is an unordered list of the IterationControls.
// in addition, the latestLastStep of every ControlGroup is tracked.
// the auto_ptr interface signifies that this list owns the IterationControl, although it is
// managed simply by virtue of the list being "destructive".

void IterationManager::
insertIterationControl(auto_ptr<IterationControl> control)
{
    // check to see if this control belongs to an existing group.
    // the casting ugliness is all to get the timestamp into a void*
    const ControlGroup* group = controlGroups_.find(findFirstStepGroup, 
        const_cast<TimeStamp*> (&(control->firstStep())));
    if (group) {
        // add this IterationControl to the existing group
        ControlGroup* theGroup = const_cast<ControlGroup*> (group);
        theGroup->insertControl(control);
    }
    else {
        // otherwise make a new group containing this control and insert it
        controlGroups_.insert(new ControlGroup(control));
    }
}

// actually insert a new IterationControl into a ControlGroup.  ensure that the latestLastStep
// is kept up to date.  assume ownership of the ptr.
// all controls in a group are assumed to have the same first step, set in the ctor.

void IterationManager::ControlGroup::
insertControl(auto_ptr<IterationControl> control)
{
    if (latestLastStep < control->lastStep())
        latestLastStep = control->lastStep();
    controls.insert(control.release());
}

// construct a new ControlGroup containing one IterationControl

IterationManager::ControlGroup::
ControlGroup(auto_ptr<IterationControl> control) :
    firstStep(control->firstStep()), latestLastStep(control->lastStep())
{
    firstStep = control->firstStep();
    insertControl(control);
}
