/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "MaturationParser.h"
#include "MaturationTokens.h"
#include "GenArrayTokens.h"
#include "Tokens.h"

#include "MaturationManager.h"
#include "Maturation.h"
#include "globals.h"
#include "GenericArrayFactory.h"

// ctor utility fns
void
MaturationParser::initKeymap()
{
    keymap_.insert(MaturationTokens::MATRATE, matRate);
    initGenArrayKeymap(keymap_);
}

void
MaturationParser::initState()
{
    // configure the initial state with the correct id list.
    State* state = new State;
    state->context = "MaturationData";
    config(state);
    stateStack.push(state);
}

MaturationParser::MaturationParser(Parser& p, RWCString dimension)
    : GenArrayParser(p), dimension_(dimension)
{
    initState();
    initKeymap();
}

// create a generic array of the requested dimension size and
// configure the MaturationManager with it.  also add the list of
// generic ids to index every element of the array to the current
// state.

void
MaturationParser::config(State* state)
{
    GenericArrayFactory<RCPtr<Maturation> > matFactory;
    GenericArrayFactory<RCPtr<Maturation> >::GenericArrayType arraytype =
        matFactory.genericArrayType(dimension_);
    if (arraytype != matFactory.NTypes) {
        auto_ptr<MatTable> matTable(matFactory.newGenericArrayFn1(arraytype, 
            MaturationManager::defaultMaturation_));
        
        matTable->newGenericIndexList(state->ids); // create the proper list of generic ids
        MaturationManager::config(matTable); // hand off maturation storage to the manager
    }
    else {
	// unidentified dimension.  log an error, but allow this
	// parser to continue with an empty id list in the state, so
	// as to skip as much data as possible without generating
	// further messages.
	logError(pError(ParseError::BadData,
			dimension_ + " is not a valid configuration for"
			+ state->context + "; discarding data")
		 .errorMessage());
    }
}


// read in one maturation object and place it in the MaturationManager
// at every generic id region in the current list
void
MaturationParser::matRate()
{
    double rate;
    parseVar(rate);
    
    Maturation* matrate = new Maturation(rate);
    
    // iterate through the ids in the current state
    RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
    while (iter != (stateStack.top()->ids).end()) {
        GenericArrayIndex* id = *iter;
        // hand off the ptr to the migration manager
        MaturationManager::insertMaturation(matrate, *id);
        ++iter;
    }
    // pop one level of state after processing the data
    parseEnd();
}
