/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include <assert.h>
#include "MigrationManager.h"
#include "Stocks.h"
#include "Chronograph.h"
#include "Geographer.h"
#include "StockYearAgeTimeMatureArray.h"
#include "TransitionMatrix.h"
#include "Cohort.h"
#include "SystemClock.h"
#include "log.h"

// static holder for the trans mat repository.
auto_ptr<GenericArray<RCPtr<TransitionMatrix> > > MigrationManager::transitionMatrices_ 
    = auto_ptr<GenericArray<RCPtr<TransitionMatrix> > > (0);


void
MigrationManager::config(TransMatRepository* tmatrep)
{
    transitionMatrices_ = auto_ptr<GenericArray<RCPtr<TransitionMatrix> > > (tmatrep); // deletion of old matrices handled by auto_ptr
}

// it is a requirement that there be an entry for every position in the migration matrix in
// order for the model to execute

void MigrationManager::
init()
{
    int erc=0;
    if (!transitionMatrices_.get())
        erc = 1;
    else {
        RWTPtrOrderedVector<GenericArrayIndex> indices;
        transitionMatrices_->newGenericIndexList(indices);
        RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = indices.begin();
        while (iter != indices.end()) {
            GenericArrayIndex* id = *iter;
            if (! (*transitionMatrices_)[*id].operator->() )
                erc = 1;
            ++iter;
        }
        indices.clearAndDestroy();
    }
    if (erc) {
        ::LogMsg(L_Fatal, "one or more transition matrices are missing!\nexiting...\n");
    }
}


// add new transition matrices.  this is an I/O operation.  reference
// counting handles memory management: shared objects and proper
// cleanup.
void
MigrationManager::insertTransitionMatrix(const RCPtr<TransitionMatrix>& tmat,
					 const GenericArrayIndex& index)
{
    (*transitionMatrices_)[index] = tmat;
}

// migrate all cohorts.  step through the active cohorts in the
// stocks, create a generic id and extract the proper transition
// matrix, then pass it to the migrate() method in the stock.

void
MigrationManager::migrateCohorts()
{
    for (int i = 0; i < Stocks.num(); ++i) {
        CohortIter next(Stocks[i].getCohorts());
        Cohort* cohort;
        while (cohort = next()) {
            GenericArrayIndex id(cohort, 0, 0, ::SystemClock->currentYearIndex(),
                ::SystemClock->currentTimeIndex());
            const TransMatRepository& tmats = *transitionMatrices_;
            cohort->migrate(*(tmats[id]), *::SystemClock);
        }
    }
}
