/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

#include "MultiCeilingIterationControl.h"
#include "Historian.h"
#include "Fishery.h"
#include "FisheryManager.h"
#include "FixedHarvestRatePolicy.h"
#include "auto_ptr.h"
#include "log.h"
#include <math.h>

const double MultiCeilingIterationControl::ceilingTolerance_ = 0.00001;
const int MultiCeilingIterationControl::maxIterations_ = 1000;

// init for every simulation execution
void MultiCeilingIterationControl::
init()
{
    iterationCount_ = 0;
}

// the endConditionSatisfied routine looks up the total catch for the fishery in the
// current year and compares it to the quota.  since the effort scalar is manipulated
// as the control variable, it is assumed that the "constant effort" requirement is 
// always met.

bool MultiCeilingIterationControl::
endConditionSatisfied()
{
    totalCatch_ = Historian::totalCatch(fishery_, firstStep_, lastStep_);
    // test the simplest condition first
    if (iterationCount_ == 0 && !force_ && totalCatch_ <= quota_) {
        return true;   
    }
    // if the above condition is not met, then we are either iterating, or the intial
    // quota is forced (or both).  either way, from this point forward we have to try
    // to hit the ceiling right on.
    else if (fabs(totalCatch_ - quota_) / quota_ < ceilingTolerance_) {
        return true;
    }
    // we will give up eventually
    else if (iterationCount_ > maxIterations_) {
        ::LogMsg(L_Error, "maximum number of iterations exceeded setting quota in fishery %s\n",
                 fishery_->get_name());
        return true;
    }
    // falling throug means iteration will occur.  if this is the first pass, set the lastGuess_.
    if (iterationCount_ == 0) {
        // choose an arbitrary region.
        GenericArrayIndex gendex(0, fishery_->index(), 0, lastStep_.yearIndex(), lastStep_.timeIndex());
        lastGuess_ = FisheryManager::effortScalar(gendex);
    }
    ++iterationCount_;
    return false;
}


// reset the control variables.  it is a precondition of this method that endConditionSatisfied()
// has been called and most recently returned 'false', so that totalCatch_ has been properly set.

void MultiCeilingIterationControl::
resetControlVariables()
{
    // compute a new effort scalar
    double guessScalar = quota_ / totalCatch_;
    double effort = lastGuess_ * guessScalar;

    // make FisheryPolicies with the appropriate effort in appropriate locations.  they are kept 
    // by fishery, region, year, and timestep
    for (TimeStamp time=firstStep_; time <= lastStep_; ++time) {
        for (int region=0; region < Geographer::nRegions(); ++region) {
            GenericArrayIndex gendex(0, fishery_->index(), region, time.yearIndex(), time.timeIndex());
            auto_ptr<FisheryPolicy> policy(new FixedHarvestRatePolicy(effort));
            // overwrite any existing policy
            FisheryManager::insertPolicy(policy, gendex);
        }
    }
    lastGuess_ = effort;
}

