/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

// MultiCeilingIterationControl.h - IterationControl derived class which handles
// ceiling management which is cumulative over multiple timesteps, while maintaining
// a constant effort across those timesteps.

#ifndef MultiCeilingIterationControl_H
#define MultiCeilingIterationControl_H

#include "IterationControl.h"

class Fishery;


// this class handles a ceiling for one fishery over multiple timesteps.  certain assumptions
// in the implementation are made which force the beginning timestep to be the first in a year
// in which any harvest occurs for that fishery, and that the ending timestep is in the same
// year.  
//
// the target for the iteration is a cumulative quota for the fishery across the timesteps in 
// question while maintaining a constant effort scalar.

class MultiCeilingIterationControl : public IterationControl {
  public:
    MultiCeilingIterationControl(const Fishery* fishery, double quota, bool force, 
        const TimeStamp& first, const TimeStamp& last)
        : IterationControl(first, last), fishery_(fishery), quota_(quota), force_(force), 
          iterationCount_(0), lastGuess_(1.0), totalCatch_(0)
    {}

    virtual void init();
    virtual void resetControlVariables();
    virtual bool endConditionSatisfied();

    // the call to saveState is a cue that we are starting a new iteration, so we
    // reset internal counters, then call the parent class method.
    virtual void saveState(auto_ptr<const SystemState> state) 
    { iterationCount_ = 0; IterationControl::saveState(state); }

  private:
    // configured data
    const Fishery* fishery_;
    double quota_;
    bool force_;

    // internal variables
    int iterationCount_;
    double lastGuess_;
    double totalCatch_;

    // class constants
    static const double ceilingTolerance_;
    static const int maxIterations_;
};

#endif // MultiCeilingIterationControl_H