/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

// MultiCeilingScalarPolicyControl.cpp


#include "MultiCeilingScalarPolicyControl.h"
#include "MultiCeilingIterationControl.h"
#include "IterationManager.h"
#include "Chronograph.h"
#include "SystemClock.h"
#include "Historian.h"
#include "auto_ptr.h"


// valid state is timesteps ok and parent class portion ok

bool MultiCeilingScalarPolicyControl:: 
valid() const
{
    if (firstStep_ < Chronograph::firstStep()
        || lastStep_ > Chronograph::lastStep()
        || firstStep_ > lastStep_)
        return false;
    else
        return CeilingScalarPolicyControl::valid();
}

// createQuotas()
// a bit of a misnomer, instead of creating quotas, this routine actually creates IterationControls.
// however, in this case those have the effect of enforcing quotas.  one IterationControl is
// created for every year for the fishery specified in the config object passed in.

void MultiCeilingScalarPolicyControl:: 
createQuotas(const CeilingScalarPolicyControl::CeilingScalarConfig& config) const
{
    int baseStartIndex = basePeriodStart_ - Chronograph::baseYear();
    int baseEndIndex = basePeriodEnd_ - Chronograph::baseYear();
    const Fishery* fishery = config.fishery();

    for (int yr = 0; yr < config.years(); ++yr) {
        struct CeilingScalarPolicyControl::CeilingScalarConfig::YearInfo yearInfo = config.yearConfig(yr);
        if (yearInfo.active_) {
            double quota = yearInfo.scalar_ 
                * Historian::basePeriodTotalAverageCatch(fishery, baseStartIndex, baseEndIndex);
            TimeStamp first(yr, firstStep_);
            TimeStamp last(yr, lastStep_);
            
            auto_ptr<IterationControl> control(
                new MultiCeilingIterationControl(fishery, quota, yearInfo.force_, first, last));
            IterationManager::insertIterationControl(control);    
        }
    }
}
