/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// NamedPtrVec - template pointer based vector for named items, with find fn

#ifndef NAMEDPTRVEC_H
#define NAMEDPTRVEC_H

#include "pragma.h"

#include <rw/tpordvec.h>
#include "Bool.h"

/*
 * The NamedPtrVec class derives from Rogue's RWTPtrOrderedVector to
 * provide a pointer based ordered vector.  Thus, it can size itself
 * dynamically based upon insert requests.  
 *
 * Although a pointer vec by implementation, access is performed using
 * a standard array idiom.  This may change in the future.
 *
 * Each item inserted in the vector is expected to be a pointer to
 * a class containing a "name" field.  A "find" fn is provided to
 * search based on the name field and return the desired element.
 *
 * Note that one of the ctors sets up the vector from an existing
 * array.  This is done by stashing the pointers, so the data had
 * better stay in scope for the lifetime of this object.
 */

template <class T>
class NamedPtrVec : public RWTPtrOrderedVector<T> {
  public:
    NamedPtrVec() : RWTPtrOrderedVector<T>() {}
    NamedPtrVec(const int n) : RWTPtrOrderedVector<T>(n) {}
    NamedPtrVec(const NamedPtrVec<T>& v) : RWTPtrOrderedVector<T>(v) {}
    NamedPtrVec(T* data, const int n);
    ~NamedPtrVec() {}
    
    NamedPtrVec<T> & operator=(const NamedPtrVec<T>& v);

    // these operators make the vector appear to be a val vec, for
    // backwards compatibility with existing code.  possibly to be
    // removed later.
    T& operator[](int i) { return *RWTPtrOrderedVector<T>::operator[](i); }
    const T& operator[](int i) const
	{ return *RWTPtrOrderedVector<T>::operator[](i); }

    // this is also for backwards compatibility and may be removed
    int num() const { return entries(); }
    
    T* find(const char *nm) const;
};

#include <rw/compiler.h>
#ifdef RW_COMPILE_INSTANTIATE
#include "NamedPtrVec.cpp"
#endif /* compile time instantiation */

#endif
