/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "NaturalMortalityManager.h"
#include "Chronograph.h"
#include "SystemClock.h"
#include "Stocks.h"
#include "NonNeg.h"
#include "GenericDefaultArray.h"


const double NaturalMortalityManager::defaultNatmortRate_ = 0.0;
auto_ptr<NatMortRateTable> NaturalMortalityManager::natmortRateTable_ 
  = auto_ptr<NatMortRateTable>(new GenericDefaultArray<double>(defaultNatmortRate_));


// reconfigure by creating a table of default rates
void NaturalMortalityManager::
config()
{
    natmortRateTable_ = auto_ptr<NatMortRateTable>(new GenericDefaultArray<double>(defaultNatmortRate_));
}

// reconfigure with new rate table
void NaturalMortalityManager::
configNatmortRates(NatMortRateTable* newTable)
{
    natmortRateTable_ = auto_ptr<NatMortRateTable>(newTable);	// auto_ptr deletes any old table
}

void NaturalMortalityManager::insertNatmortRate(double natmortRate, const GenericArrayIndex& index)
{
    (*natmortRateTable_)[index] = natmortRate;
}


// simple mortality.  loop through all active cohorts (obtained from Stocks) and regions.
// apply mortality scalar if abundance and scalar are non-zero.

void 
NaturalMortalityManager::takeNaturalMortality() 
{
    for (int s=0; s < Stocks.num(); ++s) {
        CohortList& cohorts = Stocks[s].getCohorts();
        CohortList::iterator cohortPtr = cohorts.begin();
        while (cohortPtr != cohorts.end()) {
            (*cohortPtr)->clearNaturalMortality();
            for (int region=0; region < Geographer::nRegions(); ++region) {
                const double cohortAbd = (*cohortPtr)->abundance(region);
                if (cohortAbd != 0.0) {
                    GenericArrayIndex id(*cohortPtr, 0, region, *::SystemClock);
                    double mortRate =  NaturalMortalityManager::natmortRate(id);
                    if (mortRate != 0.0) {
                        double mort = cohortAbd * mortRate;
                        (*cohortPtr)->naturalMortality(region) = mort;
                        (*cohortPtr)->abundance(region) = cohortAbd - mort;
                    }
                }
            }
            ++cohortPtr;
        }
    }
}

