/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "NaturalMortalityParser.h"
#include "NatMortTokens.h"

#include "NaturalMortalityManager.h"
#include "GenericArrayFactory.h"


// ctor utility fns
void
NaturalMortalityParser::initKeymap()
{
    keymap_.insert(NatMortTokens::NATMORTRATE, natmortRate);
    initGenArrayKeymap(keymap_);
}

void
NaturalMortalityParser::initState()
{
    // configure the initial state with the correct id list.
    State* state = new State;
    state->context = "NaturalMortalityData";
    configNatmortRate(state);
    stateStack.push(state);
}

NaturalMortalityParser::NaturalMortalityParser(Parser& p, RWCString dimension)
    : GenArrayParser(p), dimension_(dimension)
{
    initState();
    initKeymap();
}

// create a generic array for natural mortality rate data of the requested
// dimension size and configure the NaturalMortalityManager with it.  also add
// the list of generic ids to index every element of the array to the
// current state.

void
NaturalMortalityParser::configNatmortRate(State* state)
{
    GenericArrayFactory<double> natmortFactory;
    GenericArrayFactory<double>::GenericArrayType arraytype = natmortFactory.genericArrayType(dimension_); 
    if (arraytype != natmortFactory.NTypes) {
        NatMortRateTable* natmortRateTable 
            = natmortFactory.newGenericArrayFn1(arraytype, NaturalMortalityManager::defaultNatmortRate_);  
        NaturalMortalityManager::configNatmortRates(natmortRateTable); // hand off natmort storage to the manager
        natmortRateTable->newGenericIndexList(state->ids); // create the proper list of generic ids
    }
    else {
        // unidentified dimension.  log an error, but allow this
        // parser to continue with an empty id list in the state, so
        // as to skip as much data as possible without generating
        // further messages.
        logError(pError(ParseError::BadData,
            dimension_ + " is not a valid configuration for"
            + state->context + "; discarding data")
            .errorMessage());
    }
}


// read in one natmort rate object and place it in the NaturalMortalityManager
// at every generic id region in the current list
void
NaturalMortalityParser::natmortRate()
{
    double rate;
    parseVar(rate);
    
    // iterate through the ids in the current state
    RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
    while (iter != (stateStack.top()->ids).end()) {
        GenericArrayIndex* id = *iter;
        // hand off the ptr to the migration manager
        NaturalMortalityManager::insertNatmortRate(rate, *id);
        ++iter;
    }
    // pop one level of state after processing the data
    parseEnd();
}
