/**********************************************************************
  Columbia Basin Research Software Library Tools
  Copyright (C) 1998,1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

#ifndef Parser_H
#define Parser_H

// Parser class.  A base class for a hierarchical token-based
// recursive descent parser.  Provides basic fns for token retrieval
// and parsing into simple data types.  Also provides the fundamental
// mechanism for processing a file, the fn parse() which loops over
// the keyword tokens in the file and calls the (pure virtual) fn
// processKeyword() to process each one.  Inherits from
// FreeFormReader.  The default behavior is to parse keywords until
// the end of the file, outputting errors as they occur.  This class
// may throw ParseError exceptions (to indicate file not found, for
// example.)

#include "pragma.h"
#include <rw/dvec.h>

#include "FreeFormReader.h"
#include "ParseError.h"
#include "Range.h"


// This pure virtual class can be derived from to provide custom error
//  logging.
class ParserErrorLogger {
public:
    virtual void logError( const RWCString& msg ) = 0;
};


class Parser : public FreeFormReader {
  public:
    Parser(const RWCString& fileName,
           const char* separators=0, const char* operators="[];:*");
    Parser(const RWCString& fileName, ParserErrorLogger *pel,
           const char* separators=0, const char* operators="[];:*");
    virtual ~Parser() {}

    virtual void parse();

    // log an error message - customizeable
    void logError(const RWCString& msg) const;

    // Set a new custom error logger.
    void setErrorLogger( ParserErrorLogger *pel ) {
        customErrorLogger = pel;
    };
    
  protected:
    virtual void processKeyword(const RWCString&) = 0;

    // parsing fns
    void parseVar(RWCString&);
    void parseVar(int&);
    void parseVar(long&);
    void parseVar(float&);
    void parseVar(double&);
    void parseVar(DoubleVec&);
    void parseVar(Range<int>&);
    
    // a fn to retreive the current "context", also for error reporting
    virtual RWCString context() const
    { return ""; }

    // log an error message - customizeable
    // NOTE destructor will not destroy *customErrorLogger, if any.
    ParserErrorLogger *customErrorLogger;

    // a few shorthand utility fns for error processing
    static ParseError parseOk() { return ParseError::Ok; }
    ParseError pError(ParseError::Erc err,
              const RWCString& msg,
              const RWCString& tok="",
              const RWCString& context="") const
    { return ParseError(fileName(), lineNumber(), err, msg, tok, context); }
};

#endif // Parser_H

