/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Dave Salinger, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

// Search.cpp

#include "Search.h"
#include <math.h>

#ifdef DEBUG
#include <iostream.h>
#endif // DEBUG

Search::Search(double kk0, double ft, double xt, int maxIt)
: kk(kk0), ftol(ft), xtol(xt), maxIter(maxIt),
xx(1.0), xPrev(-.01), fPrev(0.0), xMax(10.0), xMin(0.0),
iter(0), xMaxTested(false), stop_code(0)
{
}

double Search::
nextGuess(double ff)
{
    // Routine to perform one iteration of a safeguarded secant method
    // for solving ff(xx) = kk. 
    // ff is assumed monotonic increasing ( and that ff(0)=0 ).
    // Safeguarding is done by a bisection method.
    // xMax is initially assumed to be an untested estimate.
    
    double xNext;
    ++iter;
    setBounds(ff);
    
    // calculate next guess
    if (kk == 0.0)              // ff(0) = 0 by definition
        return(xx=0.0);

    if (iter == 1) {
        xNext = kk/ff;
    }
    else {
#ifdef DEBUG
//        cout << ff << ", " << fPrev << ", " << ff-fPrev << endl;
#endif

        xNext = xx - (ff - kk)*(xx - xPrev)/(ff - fPrev);  // need previous
    }	                                                  // TWO  x and f. 
    xPrev = xx;
    fPrev = ff;
#ifdef DEBUG
//    cout << "xNext = " << xNext << endl;
#endif
    xx = safeguard(xNext);
    return (xx);            
}

// routine to set initial guess
void Search::
initialGuess(double xx0)
{
    xx = xx0;
}

// routine returns a true when finished time to stop
bool Search::
done(double ff)
{
    // check for stopping criteria 
    if (!stop_code) {
        // if kk = 0.0, then ff = 0.0 is the only valid stop condition
        if (kk == 0.0) {
            if (ff == 0.0)                stop_code = SUCCESS;
            if (iter > 1)
                throw;          // ff(0)=0 precondition violated
        }
        else if (fabs(ff - kk)/kk < ftol) stop_code = SUCCESS;
//        else if (fabs(xx - xPrev) < xtol) stop_code = XTOL;
        else if (iter >= maxIter)         stop_code = MAXITER;

        // check that:  ff != 0
        // for harvest fn, this means initial catch was 0, and a ceiling cannot be forced
        else if (iter == 0 && ff == 0.0) stop_code = INITF;

        // check that:  ff - fPrev != 0
        // for harvest fn, this means we ran out of fish
        else if (fabs(ff - fPrev)/kk <= .05*ftol) stop_code = FSTALL;
    }
    return (stop_code > 0);
}

// routine to set safeguarding bounds  (assumes ff is monotonic increasing)
void Search::
setBounds(double ff)
{
    if (ff > kk) {
        xMax = xx;
        xMaxTested = true;
    }
    else
        xMin = xx;
}

// routine to safeguard (by bisection method).
double Search::
safeguard(double xNext)
{
    //routine expects: certain xMin
    //                 uncertain xMax if xMaxTested = false.
    
    if (xNext < xMin	|| (xMaxTested  && (xNext >= xMax)))
        //  || fabs(xx - xNext) >= 0.5*(xMax - xMin))))
        xNext = (xMax + xMin)/2;
    return (xNext);
}
