/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Dave Salinger, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

// Search.h - class for searching parameter space

#ifndef Search_H
#define Search_H

#include "Bool.h"

// Search class.  
// The usage is to create a search object s, supplying the target value, the f tolerance,
// the x tolerance, and the maximum iterations in the ctor.  An external function (y=f(x)) 
// should be executed, then s.initialGuess(x) called.  Thereafter, an external loop should
// be executed which calls the done() and nextGuess() routines as appropriate.  
// The stopCode should be checked to see if the search was successfull.  The 
// following is a sample form for the calling code:
//
//   Search search(kk0, ft, xt, maxIt);
//   y=f(x);
//   search.initialGuess(x)
//   while (!search.done(y)) {
//     x=search.nextGuess(y);
//     y=f(x);
//   }
//   if (search.stopCode() == Search::SUCCESS)
//     ... 
//
// In this way the proper x is discovered for which f yields the target value.  
// f is assumed monotonic increasing and that f(0)=0.  Violation of this will
// throw an exception.
// The initial guess may not be 0.0, or the search will terminate immediately.
// See also further comments in Search.cpp.


class Search {
  public:
    // ctor params are target value, f tolerance, x tolerance, and max iterations
    Search(double kk0, double ft, double xt, int maxIt);
    
    double nextGuess(double ff);
    void initialGuess(double xxx);
    bool done(double ff);
    int stopCode() const { return stop_code; }
    
    enum StopCodes { SUCCESS=1, XTOL=2, MAXITER=3, FSTALL=4, INITF=5 };

  private:
    bool fstop(double ff) const;
    void setBounds(double ff);
    double safeguard(double xNext);
    
    double kk;			// function target value
    double xx;			// most recent guess
    double xPrev;		// previous guess
    double fPrev;		// previous function value
    double xMax;		// upper bound on xx (initially an untested estimate)
    double xMin;		// lower bound on xx
    int iter;			// iteration count
    bool xMaxTested;		// whether the current xMax has been tested
    int stop_code;		// reason for search termination
    
    const int maxIter;		// maximum number of iterations
    const double ftol;		// tolerance on f stop
    const double xtol;          // tolerence on x stop
};

#endif // Search_H

