/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "SpawningManager.h"
#include "CohortManager.h"
#include "StockList.h"
#include "Production.h"
#include "ProductionFunction.h"
#include "Stocks.h"
#include "SystemClock.h"
#include "GenericDefaultArray.H"


// empty table by default
auto_ptr<ProductionTable> SpawningManager::productionTable_ 
  = auto_ptr<ProductionTable>(new GenericDefaultArray<auto_ptr<Production> >(auto_ptr<Production>(0)));


// reconfigure with a default (empty) table

void SpawningManager::
config()
{
    productionTable_ = 
        auto_ptr<ProductionTable>(new GenericDefaultArray<auto_ptr<Production> >(auto_ptr<Production>(0)));
}

// reconfigure with a new table

void SpawningManager::
configProduction(auto_ptr<ProductionTable> newTable)
{
    productionTable_ = newTable;
}


void SpawningManager::
insertProductionFunction(auto_ptr<const ProductionFunction> prodFn, const GenericArrayIndex& gendex)
{
    // inserting a new function requires checking to see if a Production object already
    // exists at this gendex, and creating one if it does not.
    auto_ptr<Production>& currentProdPtr = (*productionTable_)[gendex];
    if (currentProdPtr.get() == 0) {
        // reseat the ptr to a new object
        currentProdPtr.reset(new Production);
    }
    // insert the fn into the production object (which is still in the table)
    currentProdPtr->insertProductionFunction(prodFn);
}

// call the spawning process from the table for each stock.

void 
SpawningManager::spawnCohorts() 
{
    for (int i = 0; i < Stocks.num(); ++i) {
        for (unsigned region=0; region < Geographer::nRegions(); ++region) {
            GenericArrayIndex gendex(*::SystemClock);
            gendex.stockIndex = i;
            gendex.regionIndex = region;
            const Production* production = (*productionTable_)[gendex].get();
            if (production) {
                double age0abd = Stocks[i].spawn(*production, region);
                if (age0abd > 0.0) {
                    CohortManager::makeNewAge0Cohorts(age0abd, region, Stocks[i], Chronograph::currentCalendarYear());
                }
            }
        }
    }
}

