/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


// Stock class implementations

#include <math.h>
#include "log.h"
#include "NonNeg.h"
#include "Stock.h"
#include "Fishery.h"
#include "Cohort.h"
#include "Geographer.h"
#include "Chronograph.h"
#include "CohortManager.h"
#include "NaturalMortalityManager.h"
#include "MaturationManager.h"
#include "DefaultCohortGenerator.h"
#include "Production.h"
#include "SystemClock.h"


// ctor

Stock::Stock(const RWCString& nm)
: name(nm), index_(0), prodType_(CohortID::P_UNKNOWN), run_(CohortID::FALL),
cohortGenerator_(0)
{
}

void Stock::config()
{
    // use the default cohort generator
    if (!cohortGenerator_.get())
        cohortGenerator_ = auto_ptr<CohortGenerator>(new DefaultCohortGenerator);
}

// find a currently active cohort; there should be at most one
Cohort* Stock::
findActiveCohort(const CohortID& id)
{
    CohortIter iter(cohorts_);
    Cohort* cohort;
    while (cohort = iter()) {
        if (*cohort == id)
            return cohort;
    }
    return 0;
}

// activate a cohort.  assume ownership.  also check to see if the cohort matches an existing one.
void Stock::
activateCohort(auto_ptr<Cohort> cohort)
{
    Cohort* existingCohort = findActiveCohort(cohort->id());
    if (existingCohort) {
        existingCohort->merge(*cohort);
        // outdated cohort destroyed by auto_ptr
    }
    else {
        // destructive list assumes duties of auto_ptr
        cohorts_.insert(cohort.release()); 
    }
}

/*
 * age all cohorts.
 */

void Stock::ageCohorts()
{
    // cohorts are aged to the current year
    int year = ::SystemClock->currentCalendarYear();
    CohortIter iter(cohorts_);
    Cohort* cohort;
    while (cohort = iter()) {
        // cohorts are instructed to set their age parameters to the 
        // proper value for the given year.  if one fails to do so 
        // (because the cohort is too old) then it is removed from
        // the active list and deleted.

        if (!cohort->setAge(year)) {
            iter.remove();
            delete cohort;
        }
    }
}

// used to determine "terminal run"
double Stock::adultMatureRegionalAbundance(unsigned regionIndex) const
{
    double abd = 0;
    
    CohortList::const_iterator cohortPtr = cohorts_.begin();
    while (cohortPtr != cohorts_.end()) {
        const Cohort* cohort = *cohortPtr;
        Age age = cohort->age();
        if (age >= cohort->stock()->adultAge() && cohort->maturity() == CohortID::MATURE) {
            abd += cohort->abundance(regionIndex);
        }
        ++cohortPtr;
    }
    return abd;
}

// production
double Stock::spawn(const Production& production, unsigned regionIndex)
{
    double escp = adultMatureRegionalAbundance(regionIndex);
    if (escp > 0.0)
        return (production.spawn(escp));
    else 
        return 0.0;
}


