/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "StockConfigParser.h"
#include "StockConfigTokens.h"
#include "Tokens.h"

#include "globals.h"
#include "Stock.h"

// ctor utility fn

void StockConfigParser::initKeymap()
{
    keymap_.insert(StockConfigTokens::STOCKNUMBER, stockNumber);
    keymap_.insert(StockConfigTokens::RUN, run);
    keymap_.insert(StockConfigTokens::PRODUCTIONTYPE, productionType);
    keymap_.insert(StockConfigTokens::MAXAGE, maxAge);
    keymap_.insert(StockConfigTokens::ADULTAGE, adultAge);
    keymap_.insert(StockConfigTokens::FIRSTHVAGE, firstHvAge);
    keymap_.insert(StockConfigTokens::ABBREV, abbrev);
    keymap_.insert(Tokens::END1, parseEndKeyword);
    keymap_.insert(Tokens::END2, parseEndKeyword);
}

StockConfigParser::StockConfigParser(Parser& p, const RWCString& name) 
    : SubParser(p), theStock_(new Stock(name))
{
    initKeymap();
}

RWCString
StockConfigParser::context() const
{
    return("Stock Configuration");
}

void StockConfigParser::
stockNumber()
{
    int num;
    parseVar(num);
    theStock_->number_ = num;
}

void StockConfigParser::
abbrev()
{
    // cast away const
    RWCString& abbrv = const_cast<RWCString&> (theStock_->abbrv);
    parseVar(abbrv);
}

void StockConfigParser::maxAge()
{
    int max;
    parseVar(max);
    // set or increase overall Age maximum
    Age::increaseMax(max);
    // set local max value in stock
    theStock_->maxAge_.assign_from_int(max);
}

void StockConfigParser::adultAge()
{
    int age;
    parseVar(age);
    theStock_->adultAge_.assign_from_int(age);
}

void StockConfigParser::firstHvAge()
{
    int age;
    parseVar(age);
    theStock_->firstHarvestAge_.assign_from_int(age);
}


void StockConfigParser::
run()
{
    RWCString run;
    parseVar(run);
    run.toLower();
    if (run == "fall")
        theStock_->run_ = CohortID::FALL;
    else if (run == "spring")
        theStock_->run_ = CohortID::SPRING;
    else {
        // unrecognized run type
     	throw pError(ParseError::BadData, "unrecognized run type", run);
   }
}


void StockConfigParser::
productionType()
{
    RWCString prodType;
    parseVar(prodType);
    prodType.toLower();
    if (prodType == "wild")
        theStock_->prodType_ = CohortID::WILD;
    else if (prodType == "hatchery")
        theStock_->prodType_ = CohortID::HATCHERY;
    else {
        // unrecognized production type
     	throw pError(ParseError::BadData, "unrecognized production type", prodType);
   }
}

void StockConfigParser::
parseEndKeyword()
{
    // commit this Stock
    Stocks.append(theStock_.get());
    theStock_->index(Stocks.num() - 1);
    theStock_.release();
    SubParser::parseEndKeyword();
}
