/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/


#include "TransMatParser.h"
#include "TransMatTokens.h"

#include "MigrationManager.h"
#include "TransitionMatrix.h"
#include "GenericArrayFactory.h"
#include "RCPtr.h"
#include "globals.h"


// ctor
TransMatParser::TransMatParser(Parser& p, RWCString dimension)
: GenArrayParser(p), dimension_(dimension)
{
    initState();
    initKeymap();
}

// ctor utility fn
void
TransMatParser::initKeymap()
{
    keymap_.insert(TransMatTokens::DATA, transMatData);
    initGenArrayKeymap(keymap_);
}

// configure the initial state with the correct id list.
void
TransMatParser::initState()
{
    State* state = new State;
    state->context = "TransitionMatrix";
    config(state);
    stateStack.push(state);
}

// create a generic array of the requested dimension size and
// configure the MigrationManager with it.  also add the list of
// generic ids to index every element of the array to the current
// state.
void
TransMatParser::config(State* state)
{
    GenericArrayFactory<RCPtr<TransitionMatrix> > tmFactory;
    GenericArrayFactory<RCPtr<TransitionMatrix> >::GenericArrayType arraytype
        = tmFactory.genericArrayType(dimension_);
    if (arraytype != tmFactory.NTypes) {
        TransMatRepository* tmTable = tmFactory.newGenericArrayFn0(arraytype);
        MigrationManager::config(tmTable); // hand off transmat storage to the manager
        tmTable->newGenericIndexList(state->ids); // create the proper list of generic ids
    }
    else {
        // unidentified dimension.  log an error, but allow this
        // parser to continue with an empty id list in the state, so
        // as to skip as much data as possible without generating
        // further messages.
        logError(pError(ParseError::BadData,
            dimension_ + " is not a valid configuration for"
            + state->context + "; discarding data")
            .errorMessage());
    }
}

// read in one transition matrix and place it in the MigrationManager
// at every generic id region in the current list
void
TransMatParser::transMatData()
{
    int nlocs = Geographer::nRegions();
    // create reference counted transition matrix
    RCPtr<TransitionMatrix> transMatPtr(new TransitionMatrix(nlocs));
    // grab a view of the matrix
    TransitionMatrix& matrix = *transMatPtr;
    
    // read in the values
    for (int i=0; i < nlocs; ++i) {
        for (int j=0; j < nlocs; ++j) {
            parseVar(matrix(i, j));
        }
    }
    // iterate through the ids in the current state
    RWTPtrOrderedVector<GenericArrayIndex>::iterator iter = (stateStack.top()->ids).begin();
    while (iter != (stateStack.top()->ids).end()) {
        GenericArrayIndex* id = *iter;
        // hand off the ptr to the migration manager
        MigrationManager::insertTransitionMatrix(transMatPtr, *id);
        ++iter;
    }
    // pop one level of state after processing the data
    parseEnd();
}
