/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1998  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/




/*
 * log.h
 *
 *	The LogLevel class, which provides log window capabilities.
 * The function LogMsg, which is the primary interface for this
 * utility and takes a LogLevel parameter, simply calls the public member
 * function display_log_message for the LogLevel passed in.
 *
 */

#ifndef C2_LOG_H
#define C2_LOG_H
#include <string.h>
#include <stdio.h>
#include <stdarg.h>
#include <rw/tpslist.h>
#include "fstream.h"

#include "Bool.h"

#ifndef WIN32
extern "C" {
    #include <errno.h>
    extern int errno;
    extern int sys_nerr;
    extern char *sys_errlist[];
}
#endif // WIN32

/* 
 * The constructor for LogLevel requires at least two parameters, the first,
 *	a boolean, specifies whether that LogLevel is "set" at init time.
 */

class LogLevel {
  public:
    LogLevel(Bool, char *, char * = 0, Bool = FALSE, Bool = TRUE,
	     Bool = FALSE);
    ~LogLevel();

//    static void display_log_message(char *);
    static void display_log_message(const char *);
    static void config(const char* filename); // open logfile
    static void cleanup(); // cleanup logfile

    Bool is_set() const { return flag; }
    Bool set()  { if (is_settable()) {flag=TRUE;}  return flag; }
    Bool clear(){ if (is_settable()) {flag=FALSE;} return flag; }

    int is_fatal() const { return fatal; }
    int is_settable() const { return user_settable; }
    int forces_show() const { return force; }
    const char *get_name() const { return name; }
    const char *get_prefix() const { return prefix; }

    // These two operators are mostly bogus, added to allow use of the list.
    int operator==(const LogLevel &rhs) const { return this == &rhs; }
    int operator<(const LogLevel &rhs)  const { return this <  &rhs; }

  private:

    static ofstream logfile;

    //Bool set(Bool v) { return flag = v; }

    // Note that because there is no copy constructor, passing by value is
    // *NOT* allowed!
    LogLevel(const LogLevel &);				// Prohibit its use.
    const LogLevel &operator=(const LogLevel &);	// Prohibit its use.

    Bool flag;
    Bool user_settable;
    Bool force;
    Bool fatal;

    char *name;
    char *prefix;
};

// These names don't fit in with the traditional naming scheme for
// backwards compatability.

extern LogLevel L_Error;
extern LogLevel L_Warning;
extern LogLevel L_Msg;
extern LogLevel L_IOMsg;
extern LogLevel L_DbgMsg;
extern LogLevel L_Fatal;
extern LogLevel L_Force;

/*
 * int LogTest(const LogLevel &l) A fast in-line test of a LogLevel to see
 *	if it is set.
 */
inline int LogTest(const LogLevel &l)
{
    return l.is_set();
}

/* LogBuf exists because current C++ compilers can handle temporary
 * arrays in inlined functions.  Go figure.  I don't like this way of
 * doing it (with a global variable) but it's tradition by now. -Tom
 */

extern char LogBuf[];

/* LogMsg was originally inlined, but this Sun's C++ compiler can't
 * handle optimizing and resolving an inlined function using stdargs,
 * so it's been moved to log.cc.
 */

extern void LogMsg(const LogLevel &l, const char *format, ...);


#endif /* C2_LOG_H */
