/**********************************************************************
  Pacific Coast Harvest Model - a coastal piscatorial simulation
  Copyright (C) 1999  Troy Frever, Columbia Basin Research, University of Washington
     
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
     
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  For questions regarding this program, please contact:

  Troy Frever or Jim Norris
  UW Columbia Basin Research
  Box 358218
  University of Washington
  Seattle, WA  98195
  troy@u.washington.edu

**********************************************************************/

// VulnerabilityTable.h - a reference counted GenericArray of bool


#ifndef VulnerabilityTable_H
#define VulnerabilityTable_H

#include "GenericArray.h"
#include "auto_ptr.h"
#include "RCObject.h"
#include "RCPtr.h"

// VulnerabilityTable is a GenericArray of bool.  It is a reference counted class,
// so copies may be made freely.  It is initialized with an existing table
// which it assumes ownership of (shown via auto_ptr).  
// 
// This class is implemented as a Decorator for GenericArray, supporting the interface
// of that class, but adding the reference counting.  Unfortunately, this requires
// two layers of indirection (one for the decorator, and another for the internal
// RCPtr).  Because the RCPtr is a data member of this class, all the magic happens
// through the use of the default copy ctor.


class VulnerabilityTable {
  public:
    // initialization via auto_ptr shows that this class assumes ownership.  
    // actual memory management is handled via RCPtr

    VulnerabilityTable(auto_ptr<GenericArray<bool> > table) : ptr_(new RCTable(table))
    {}
//    VulnerabilityTable()
//    {}
    ~VulnerabilityTable()
    {}
    // default copy ctor works properly, since the only data member is an RCPtr

    const bool& operator[](const GenericArrayIndex& gendex) const 
    { return (**ptr_)[gendex]; }
    bool& operator[](const GenericArrayIndex& gendex)
    { return (**ptr_)[gendex]; }

    void newGenericIndexList(RWTPtrOrderedVector<GenericArrayIndex>& list) const 
    { (*ptr_)->newGenericIndexList(list); }

  private:
      class RCTable : private RCObject {
        public:
          friend class RCPtr<RCTable>;
          
          RCTable(auto_ptr<GenericArray<bool> > table) : table_(table.release())
          {}
          ~RCTable()
          { delete table_; }

          GenericArray<bool>* operator->() const
          { return table_; }

          GenericArray<bool>& operator*() const
          { return *table_; }

          GenericArray<bool>* table_;
      };
      RCPtr<RCTable> ptr_;
};

#endif // VulnerabilityTable_H